#ifndef CM3_REGISTERS_H
#define CM3_REGISTERS_H

/*{ cm3_registers.h ******************************************************
 
 Structure definitions of basic CortexM3-registers.
 
 written by Gregor Rebel 2012
 
}*/
//{ Defines/ TypeDefs ****************************************************


//} Defines
//{ Includes *************************************************************

 // Basic set of helper functions
#include "ttc_basic.h"

#define USE_STDPERIPH_DRIVER
#include "stm32f10x.h"
#include "core_cm3.h"

//} Includes
//{ cm3 Registers ******************************************************



typedef struct { // SCB_CPUID_t CPUID register -> PM0056 p.131
    unsigned Revision    : 4;  // Revision number (patch release) (0x0 = p0, 0x1 = p1)
    unsigned PartNo      : 12; // part number of the processor (0xc23 = Cortex-M3)
    unsigned Constant    : 4;  // reads as 0xf
    unsigned Variant     : 4;  // variant number (0x1 = r1, 0x2 = r2)
    unsigned Implementer : 8; // Implementer code (0x41 = ARM)
} __attribute__((__packed__)) (SCB_CPUID_t);
typedef struct { // SCB_ICSR_t  Interrupt Control and State Register -> PM0056 p.132
    unsigned VECTACTIVE  : 9;  // Active vector; Contains the active exception number: 0: Thread mode/Other values: The exception number(1) of the currently active exception. Note: Subtract 16 from this value to obtain the IRQ number required to index into the Interrupt Clear-Enable, Set-Enable, Clear-Pending, Set-Pending, or Priority Registers, see Table 5 on page 19 in PM0056.
    unsigned reserved1   : 2;
    unsigned RETOBASE    : 1;  // Return to base level =0: There are preempted active exceptions to execute/ =1: There are no active exceptions, or the currently-executing exception is the only active exception.
    unsigned VECTPENDING : 10; // Pending vector; Indicates the exception number of the highest priority pending enabled exception. 0: No pending exceptions
    unsigned ISRPENDING  : 1;  // Interrupt pending flag, excluding NMI and Faults =0: Interrupt not pending/ =1: Interrupt pending
    unsigned DEBUG       : 1;  // This bit is reserved for Debug use and reads-as-zero when the processor is not in Debug.
    unsigned reserved2   : 1;
    unsigned PENDSTCLR   : 1;  // SysTick exception clear-pending bit. Write: 0: No effect/1: Removes the pending state from the SysTick exception.; This bit is write-only. On a register read its value is unknown.
    unsigned PENDSTSET   : 1;  // SysTick exception set-pending bit. Write: 0: No effect/1: Change SysTick exception state to pending; Read: 0: SysTick exception is not pending/1: SysTick exception is pending
    unsigned PENDSVCLR   : 1;  // PendSV clear-pending bit. Write: 0: No effect/1: Removes the pending state from the PendSV exception.
    unsigned PENDSVSET   : 1;  // PendSV set-pending bit. Write: 0: No effect/1: Change PendSV exception state to pending.; Read: 0: PendSV exception is not pending/ 1: PendSV exception is pending; Writing 1 to this bit is the only way to set the PendSV exception state to pending.
    unsigned reserved3   : 2;
    unsigned NMIPENDSET  : 1;  // NMI set-pending bit. Write: 0: No effect/1: Change NMI exception state to pending.; Read: 0: NMI exception is not pending/1: NMI exception is pending
} __attribute__((__packed__)) (SCB_ICSR_t);
typedef struct { // SCB_VTOR_t  Vector table offset register
    unsigned reserved1   : 2;
    unsigned TableOffset : 29; // offset address of table base from memory address 0x0 (bits 8:0 must be kept zero!)
    unsigned TableInSRAM : 1;  // =1: vector table is in SRAM; =0: VeectorTable is in Code
} __attribute__((__packed__)) (SCB_VTOR_t);
typedef struct { // SCB_AIRCR_t Application interrupt and reset control register
    unsigned VECTRESET      : 1;  // Reserved for Debug use. This bit reads as 0. When writing to the register you must write 0 to this bit, otherwise behavior is unpredictable.
    unsigned VECTCLRACTIVE  : 1;  // Reserved for Debug use. This bit reads as 0. When writing to the register you must write 0 to this bit, otherwise behavior is unpredictable.
    unsigned SYSRESETREQ    : 1;  // System reset request. This is intended to force a large system reset of all major components except for debug. write 1: Asserts a signal to the outer system that requests a reset.
    unsigned reserved1      : 5;
    unsigned PRIGROUP       : 3;  // Interrupt priority grouping field. This field determines the split of group priority from subpriority, see Binary point on PM0056 p.135.
    unsigned reserved2      : 4;
    unsigned ENDIANESS      : 1;  // Data endianness bit. Reads as 0 = Little-endian
    unsigned VECTKEYSTAT    : 16; // VECTKEY[15:0] Register key. Reads as 0xFA05. On writes, write 0x5FA to VECTKEY, otherwise the write is ignored.
} __attribute__((__packed__)) (SCB_AIRCR_t);
typedef struct { // SCB_SCR_t   System control register
    unsigned reserved1    : 1;
    unsigned SLEEPONEXIT  : 1;   // =1: Enter sleep, or deep sleep, on return from an interrupt service routine.
    unsigned SLEEPDEEP    : 1;   // =1: use deep sleep as low power mode
    unsigned reserved2    : 1;
    unsigned SEVEONPEND   : 1;   // =1: Enabled events and all interrupts, including disabled interrupts, can wakeup the processor; only enabled events or interrupts otherwise
    unsigned reserved3    : 27;
} __attribute__((__packed__)) SCB_SCR_t;
typedef struct { // SCB_CCR_t   Configuration and control register
    unsigned NONBASETHRDENA : 1;  // =1: Processor can enter Thread mode from any level under the control of an EXC_RETURN; =0: only when no exception is active
    unsigned USERSETMPEND   : 1;  // =1: Enables unprivileged software access to the STIR, see Software trigger interrupt register (NVIC_STIR) on page 127
    unsigned reserved1      : 1;
    unsigned UNALIGN_TRP    : 1;  // =1: Trap unaligned halfword and word accesses
    unsigned DIV_0_TRP      : 1;  // =1: Enable faulting or halting when the processor executes an SDIV or UDIV instruction with a divisor of 0
    unsigned reserved2      : 3;
    unsigned BFHFNMIGN      : 1;  // =1: Handlers running at priority -1 and -2 ignore data bus faults caused by load and store instructions
    unsigned STKALIGN       : 1;  // =1: stack is 4-byte aligned; =1: stack is 8-byte aligned
    unsigned reserved3      : 22;
} __attribute__((__packed__)) SCB_CCR_t;
typedef struct { // SCB_SHPR1_t System handler priority register 1
    unsigned PRI_4_MemoryManagementFault : 8; // Priority of system handler 4, memory management fault
    unsigned PRI_5_BusFault              : 8; // Priority of system handler 5, bus fault
    unsigned PRI_6_UsageFault            : 8; // Priority of system handler 6, usage fault
    unsigned reserved1                   : 8;
} __attribute__((__packed__)) (SCB_SHPR1_t);
typedef struct { // SCB_SHPR2_t System handler priority register 2
    unsigned reserved1                   : 24;
    unsigned PRI_11_SVCall               : 8; // Priority of system handler 11, SVCall
} __attribute__((__packed__)) (SCB_SHPR2_t);
typedef struct { // SCB_SHPR3_t System handler priority register 3
    unsigned reserved1                   : 16;
    unsigned PRI_15_SysTickException     : 8; // Priority of system handler 15, SysTick exception
    unsigned PRI_14_PendSV               : 8; // Priority of system handler 14, PendSV
} __attribute__((__packed__)) (SCB_SHPR3_t);
typedef struct { // SCB_SHCSR_t System handler control and state register (->PM0056 p.140)
    unsigned MEMFAULTACT    : 1; // Memory management fault exception active bit, reads as 1 if exception is active
    unsigned BUSFAULTACT    : 1; // Bus fault exception active bit, reads as 1 if exception is active
    unsigned reserved1      : 1;
    unsigned USGFAULTACT    : 1; // Usage fault exception active bit, reads as 1 if exception is active
    unsigned reserved2      : 3;
    unsigned SVCALLACT      : 1; // SVC call active bit, reads as 1 if SVC call is active
    unsigned MONITORACT     : 1; // Debug monitor active bit, reads as 1 if Debug monitor is active
    unsigned reserved3      : 1;
    unsigned PENDSVACT      : 1; // PendSV exception active bit, reads as 1 if exception is active
    unsigned SYSTICKACT     : 1; // SysTick exception active bit, reads as 1 if exception is active
    unsigned USGFAULTPENDED : 1; // Usage fault exception pending bit, reads as 1 if exception is pending
    unsigned MEMFAULTPENDED : 1; // Memory management fault exception pending bit, reads as 1 if exception is pending
    unsigned BUSFAULTPENDED : 1; // Bus fault exception pending bit, reads as 1 if exception is pending
    unsigned SVCALLPENDED   : 1; // SVC call pending bit, reads as 1 if exception is pending
    unsigned MEMFAULTENA    : 1; // Memory management fault enable bit, set to 1 to enable
    unsigned BUSFAULTENA    : 1; // Bus fault enable bit, set to 1 to enable
    unsigned USGFAULTENA    : 1; // Usage fault enable bit, set to 1 to enable
    unsigned reserved4      : 13;
} __attribute__((__packed__)) SCB_SHCSR_t;
typedef struct { // SCB_CFSR_MMFSR_t  Memory Management Fault Status Register (->PM0056 p.142)
    unsigned IACCVIOL  : 1;   // =1: Instruction access violation flag
    unsigned DACCVIOL  : 1;   // =1: Data access violation flag
    unsigned reserved1 : 1;
    unsigned MUNSTKERR : 1;   // =1: Memory manager fault on unstacking for a return from exception
    unsigned MSTKERR   : 1;   // =1: Memory manager fault on stacking for exception entry
    unsigned reserved2 : 2;
    unsigned MMARVALID : 1;   // =1: Memory Management Fault Address Register (MMAR) is valid
} __attribute__((__packed__)) SCB_CFSR_MMFSR_t;
typedef struct { // SCB_CFSR_BFSR_t   Bus Fault Status Register (->PM0056 p.142)
    unsigned IBUSERR     : 1;  // =1: Instruction bus error
    unsigned PRECISEERR  : 1;  // =1: Precise data bus error (faulting address in BFAR)
    unsigned IMPRECISERR : 1;  // =1: Imprecise data bus error (no stack frame or fault address known)
    unsigned UNSTKERR    : 1;  // =1: Bus fault on unstacking for a return from exception (original stack stil present)
    unsigned STKERR      : 1;  // =1: Bus fault on stacking for exception entry
    unsigned reserved1   : 2;
    unsigned BFARVALID  : 1;   // =1: Bus Fault Address Register (BFAR) is valid
} __attribute__((__packed__)) SCB_CFSR_BFSR_t;
typedef struct { // SCB_CFSR_UFSR_t   Usage Fault Status Register (->PM0056 p.143)
    unsigned UNDEFINSTR  : 1;   // =1: Undefined instruction usage fault (processor has attempted to execute an undefined instruction)
    unsigned INVSTATE    : 1;   // =1: Invalid state usage fault (processor has attempted to execute an instruction that makes illegal use of the EPSR)
    unsigned INVPC       : 1;   // =1: Invalid PC load usage fault, caused by an invalid PC load by EXC_RETURN
    unsigned NOCP        : 1;   // =1: No coprocessor usage fault
    unsigned reserved1   : 4;
    unsigned UNALIGNED   : 1;   // =1: processor has made an unaligned memory access
    unsigned DIVBYZERO   : 1;   // =1:  processor has executed an SDIV or UDIV instruction with a divisor of 0
    unsigned reserved2   : 6;
} __attribute__((__packed__)) SCB_CFSR_UFSR_t;
typedef struct { // SCB_CFSR_t Configurable fault status register (->PM0056 p.142)
    SCB_CFSR_MMFSR_t MMFSR;          // Memory Management Fault Status Register
    SCB_CFSR_BFSR_t  BFSR;           // Bus Fault Status Register
    SCB_CFSR_UFSR_t  UFSR;           // Usage Fault Status Register
} __attribute__((__packed__)) SCB_CFSR_t;
typedef struct { // SCB_HFSR_t Hard fault status register (->PM0056 p.145)
    unsigned reserved1   : 1;
    unsigned VectorTable : 1;   // =1: hardfault was caused by vector table read
    unsigned reserved2   : 28;
    unsigned Forced      : 1;   // =1: hardfault was forced by escalation of fault that could not be handled
    unsigned DebugVT     : 1;   // =1: reserved for debug use
} __attribute__((__packed__)) SCB_HFSR_t;
typedef struct { // SCB_t      System Control Block
  volatile SCB_CPUID_t CPUID;                   /*!< Offset: 0x00  CPU ID Base Register                                  */
  volatile SCB_ICSR_t  ICSR;                    /*!< Offset: 0x04  Interrupt Control State Register                      */
  volatile SCB_VTOR_t  VTOR;                    /*!< Offset: 0x08  Vector Table Offset Register                          */
  volatile SCB_AIRCR_t AIRCR;                   /*!< Offset: 0x0C  Application Interrupt / Reset Control Register        */
  volatile SCB_SCR_t   SCR;                     /*!< Offset: 0x10  System Control Register                               */
  volatile SCB_CCR_t   CCR;                     /*!< Offset: 0x14  Configuration Control Register                        */
  volatile SCB_SHPR1_t SHPR1;                   /*!< Offset: 0x18  System Handlers Priority Register 1                   */
  volatile SCB_SHPR2_t SHPR2;                   /*!< Offset: 0x1c  System Handlers Priority Register 2                   */
  volatile SCB_SHPR3_t SHPR3;                   /*!< Offset: 0x20  System Handlers Priority Register 3                   */
  volatile SCB_SHCSR_t SHCSR;                   /*!< Offset: 0x24  System Handler Control and State Register             */
  volatile SCB_CFSR_t  CFSR;                    /*!< Offset: 0x28  Configurable Fault Status Register                    */
  volatile SCB_HFSR_t  HFSR;                    /*!< Offset: 0x2C  Hard Fault Status Register                            */

  volatile u32_t DFSR;                         /*!< Offset: 0x30  Debug Fault Status Register                           */
  volatile u32_t MMFAR;                        /*!< Offset: 0x34  Mem Manage Address Register                           */
  volatile u32_t BFAR;                         /*!< Offset: 0x38  Bus Fault Address Register                            */

  // no covered by PM0056.. (copied from CMSIS/.../core_cm3.h)
  volatile u32_t AFSR;                         /*!< Offset: 0x3C  Auxiliary Fault Status Register                       */
  volatile u32_t PFR[2];                       /*!< Offset: 0x40  Processor Feature Register                            */
  volatile u32_t DFR;                          /*!< Offset: 0x48  Debug Feature Register                                */
  volatile u32_t ADR;                          /*!< Offset: 0x4C  Auxiliary Feature Register                            */
  volatile u32_t MMFR[4];                      /*!< Offset: 0x50  Memory Model Feature Register                         */
  volatile u32_t ISAR[5];                      /*!< Offset: 0x60  ISA Feature Register                                  */
} __attribute__((__packed__)) SCB_t;

typedef struct { // NVIC_Interrupts_t   interrupt lines (common to most NVIC-registers)
    unsigned IRQ00     : 1;
    unsigned IRQ01     : 1;
    unsigned IRQ02     : 1;
    unsigned IRQ03     : 1;
    unsigned IRQ04     : 1;
    unsigned IRQ05     : 1;
    unsigned IRQ06     : 1;
    unsigned IRQ07     : 1;
    unsigned IRQ08     : 1;
    unsigned IRQ09     : 1;
    unsigned IRQ10     : 1;
    unsigned IRQ11     : 1;
    unsigned IRQ12     : 1;
    unsigned IRQ13     : 1;
    unsigned IRQ14     : 1;
    unsigned IRQ15     : 1;
    unsigned IRQ16     : 1;
    unsigned IRQ17     : 1;
    unsigned IRQ18     : 1;
    unsigned IRQ19     : 1;
    unsigned IRQ20     : 1;
    unsigned IRQ21     : 1;
    unsigned IRQ22     : 1;
    unsigned IRQ23     : 1;
    unsigned IRQ24     : 1;
    unsigned IRQ25     : 1;
    unsigned IRQ26     : 1;
    unsigned IRQ27     : 1;
    unsigned IRQ28     : 1;
    unsigned IRQ29     : 1;
    unsigned IRQ30     : 1;
    unsigned IRQ31     : 1;
    unsigned IRQ32     : 1;
    unsigned IRQ33     : 1;
    unsigned IRQ34     : 1;
    unsigned IRQ35     : 1;
    unsigned IRQ36     : 1;
    unsigned IRQ37     : 1;
    unsigned IRQ38     : 1;
    unsigned IRQ39     : 1;
    unsigned IRQ40     : 1;
    unsigned IRQ41     : 1;
    unsigned IRQ42     : 1;
    unsigned IRQ43     : 1;
    unsigned IRQ44     : 1;
    unsigned IRQ45     : 1;
    unsigned IRQ46     : 1;
    unsigned IRQ47     : 1;
    unsigned IRQ48     : 1;
    unsigned IRQ49     : 1;
    unsigned IRQ50     : 1;
    unsigned IRQ51     : 1;
    unsigned IRQ52     : 1;
    unsigned IRQ53     : 1;
    unsigned IRQ54     : 1;
    unsigned IRQ55     : 1;
    unsigned IRQ56     : 1;
    unsigned IRQ57     : 1;
    unsigned IRQ58     : 1;
    unsigned IRQ59     : 1;
    unsigned IRQ60     : 1;
    unsigned IRQ61     : 1;
    unsigned IRQ62     : 1;
    unsigned IRQ63     : 1;
    unsigned IRQ64     : 1;
    unsigned IRQ65     : 1;
    unsigned IRQ66     : 1;
    unsigned IRQ67     : 1;
    unsigned reserved1 : 28;
    u32_t    reserved2[5];
} __attribute__((__packed__)) NVIC_Interrupts_t;

typedef NVIC_Interrupts_t NVIC_ISER_t; // Interrupt Set Enable Register
typedef NVIC_Interrupts_t NVIC_ICER_t; // Interrupt Clear Enable Register
typedef NVIC_Interrupts_t NVIC_ISPR_t; // Interrupt Set Pending Register
typedef NVIC_Interrupts_t NVIC_ICPR_t; // Interrupt Clear Pending Register
typedef NVIC_Interrupts_t NVIC_IABR_t; // Interrupt Active bit Register
typedef struct { // NVIC_IP_t     Interrupt Priority Register (8Bit wide)
    unsigned Priority : 4;
    unsigned reserved : 4;
} __attribute__((__packed__)) NVIC_IP_t;
typedef struct { // NVIC_STIR_t   Software Trigger Interrupt Register
    unsigned InterruptID : 8;          // software generated interrupt ID
    unsigned reserved    : 8;
} __attribute__((__packed__)) NVIC_STIR_t;
typedef struct { // NVIC_t     Nested Vector Interrupt Controller (->ST PM0056 p.119)
    volatile NVIC_ISER_t ISER;                       // Offset: 0x000  Interrupt Set Enable Register
    u32_t reserved1[24];
    volatile NVIC_ICER_t ICER;                       // Offset: 0x080  Interrupt Clear Enable Register
    u32_t reserved2[24];
    volatile NVIC_ISPR_t ISPR;                       // Offset: 0x100  Interrupt Set Pending Register
    u32_t reserved3[24];
    volatile NVIC_ICPR_t ICPR;                       // Offset: 0x180  Interrupt Clear Pending Register
    u32_t reserved4[24];
    volatile NVIC_IABR_t IABR;                       // Offset: 0x200  Interrupt Active bit Register
    u32_t reserved5[56];
    volatile NVIC_IP_t   IP[240];                    // Offset: 0x300  Interrupt Priority Register (8Bit wide)
    u32_t reserved6[644];
    volatile NVIC_STIR_t STIR;                       // Offset: 0xE00  Software Trigger Interrupt Register

} __attribute__((__packed__)) NVIC_t;

//} cm3 Registers

#endif //CM3_REGISTERS_H
