/*{ em357_gpio.c ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Low Level Driver for General Purpose Input Output (GPIO) on EM357/EM357 architecture.
 * 
}*/

#include "em357_gpio.h"


GPIOMode_TypeDef em357_map_Type(ttc_gpio_mode_e Type) {
    switch (Type) { // find best matching type for stm32-architecture

    case tgm_analog_in:
      return GPIOCFG_ANALOG;
    case tgm_input_floating:
      return GPIOCFG_IN;
    case tgm_input_pull_down:
      return GPIOCFG_IN_PUD;
    case tgm_input_pull_up:
      return GPIOCFG_IN_PUD;
    case tgm_output_open_drain:
      return GPIOCFG_OUT_OD;
    case tgm_output_push_pull:
      return GPIOCFG_OUT;
    case tgm_alternate_function_push_pull:
      return GPIOCFG_OUT_ALT;
    case tgm_alternate_function_open_drain:
      return GPIOCFG_OUT_ALT_OD;
    case tgm_alternate_function_SPECIAL_SCLK:
      return GPIOCFG_OUT_ALT_SPECIAL_SCLK ;

    default: Assert(0, ec_InvalidArgument);
    }

    return 0;
}
GPIOSpeed_TypeDef em357_map_Speed(ttc_gpio_speed_e Speed) {
    switch (Speed) { // find best matching speed for stm32-architecture
    case tgs_Min:
    case tgs_2MHz:
    case tgs_10MHz:
    case tgs_50MHz:
    case tgs_Max:   return 0;
    default: Assert(0, ec_InvalidArgument);
    }

    return 0;
}
inline void em357_loadPort_variable(em357_Port_t* Port, GPIO_em357_e GPIOx, u8_t Pin) {
        Port->GPIOx=GPIOx;
        Port->Pin  =Pin;
}
void em357_initPort_variable(em357_Port_t* Port, GPIOMode_TypeDef Type) {
  em357_initPort2_variable(Port, Type, tgs_Max);
}
void em357_initPort2_variable(em357_Port_t* Port, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) {
    em357_initPort2(Port->GPIOx, Port->Pin, Type, Speed);
}
void em357_initPort(GPIO_em357_e GPIOx, u8_t Pin, GPIOMode_TypeDef Type) {
    em357_initPort2(GPIOx, Pin, Type, tgs_Max);
}
void em357_initPort2(GPIO_em357_e GPIOx, u8_t Pin, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) {

       switch (Type) {

       case GPIOCFG_OUT_mode: {
           switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
             case (u32_t) eg_GPIOA:  halGpioConfig(PORTA_PIN(Pin), GPIOCFG_OUT); break;
             case (u32_t) eg_GPIOB:  halGpioConfig(PORTB_PIN(Pin), GPIOCFG_OUT); break;
             case (u32_t) eg_GPIOC:  halGpioConfig(PORTC_PIN(Pin), GPIOCFG_OUT); break;

             default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
           }
              break;
           }
       case GPIOCFG_OUT_OD_mode: {
       break;
    }
       case GPIOCFG_OUT_ALT_mode: {
       break;
    }
       case GPIOCFG_OUT_ALT_OD_mode: {
       break;
    }
       case GPIOCFG_ANALOG_mode: {
       break;
    }
       case GPIOCFG_IN_mode: {
       break;
    }
       case GPIOCFG_IN_PUD_mode: {
       break;
    }
       case GPIOCFG_OUT_ALT_SPECIAL_SCLK_mode: {

           switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
             case (u32_t) eg_GPIOA:  halGpioConfig(PORTA_PIN(Pin), GPIOCFG_OUT_ALT_SPECIAL_SCLK); break;
             case (u32_t) eg_GPIOB:  halGpioConfig(PORTB_PIN(Pin), GPIOCFG_OUT_ALT_SPECIAL_SCLK); break;
             case (u32_t)  eg_GPIOC:  halGpioConfig(PORTC_PIN(Pin), GPIOCFG_OUT_ALT_SPECIAL_SCLK); break;

             default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
           }

       break;
    }
       default: Assert(0, ec_InvalidArgument);

      }

}
inline BOOL em357_portGet_variable(em357_Port_t* Port) {
    return em357_portGet(Port->GPIOx, Port->Pin);
}
BOOL em357_portGet(GPIO_em357_e GPIOx, u8_t Pin) {
    u32_t Mask = (u32_t) 1 << Pin;

    //must use Registers, because hal Library does not support reading GPIOs
    switch ( (u32_t) GPIOx) {
    case (u32_t) eg_GPIOA:  {
        if ( ( (u32_t) GPIO_PAIN_REG) &  Mask)
            return TRUE;
        else
            return FALSE;
        break;
    }
    case (u32_t) eg_GPIOB:  {
        if ( ( (u32_t) GPIO_PBIN_REG) &  Mask)
            return TRUE;
        else
            return FALSE;
        break;
    }
    case (u32_t) eg_GPIOC:  {
        if ( ( (u32_t) GPIO_PCIN_REG) &  Mask)
            return TRUE;
        else
            return FALSE;
        break;
    }

    default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
    }
    return 0;
}
inline void em357_portSet_variable(em357_Port_t* Port) {
    em357_portSet(Port->GPIOx, Port->Pin);
}
void em357_portSet(GPIO_em357_e GPIOx, u8_t Pin) {

     switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
     case (u32_t) eg_GPIOA:  halGpioSet(PORTA_PIN(Pin), 1); break;
     case (u32_t) eg_GPIOB:  halGpioSet(PORTB_PIN(Pin), 1); break;
     case (u32_t) eg_GPIOC:  halGpioSet(PORTC_PIN(Pin), 1); break;

     default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
     }
}
inline void em357_portClr_variable(em357_Port_t* Port) {
    em357_portClr(Port->GPIOx, Port->Pin);
}
void em357_portClr(GPIO_em357_e GPIOx, u8_t Pin) {

    switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
    case (u32_t) eg_GPIOA:  halGpioSet(PORTA_PIN(Pin), 0); break;
    case (u32_t) eg_GPIOB:  halGpioSet(PORTB_PIN(Pin), 0); break;
    case (u32_t)   eg_GPIOC:  halGpioSet(PORTC_PIN(Pin), 0); break;

    default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
  }
}
