#ifndef EM357_GPIO_H
#define EM357_GPIO_H

/*{ em357_gpio.h ***********************************************
 
 * Written by Gregor Rebel, Sascha Poggemann 2010-2012
 *
 * Low Level Driver for General Purpose Input Output (GPIO) on EM357 architecture.
 * 
}*/
//{ defines required to include HAL-library

//}
//{ includes1

#include "ttc_basic.h"

//}includes
//{ Structures/ Enums

typedef enum { // GPIOMode_TypeDef
    GPIOCFG_OUT_mode = 0x1u,
    GPIOCFG_OUT_OD_mode = 0x5u,
    GPIOCFG_OUT_ALT_mode = 0x9u,
    GPIOCFG_OUT_ALT_OD_mode = 0xDu,
    GPIOCFG_ANALOG_mode = 0x0u,
    GPIOCFG_IN_mode = 0x4u,
    GPIOCFG_IN_PUD_mode = 0x8u,
    GPIOCFG_OUT_ALT_SPECIAL_SCLK_mode = 0xBu
} GPIOMode_TypeDef;

typedef enum { // GPIOSpeed_TypeDef
    GPIOSpeed_not_implemented= 0
} GPIOSpeed_TypeDef;

typedef enum { // GPIO_em357_e
    GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C
} GPIO_em357_e;

typedef struct { // em357_Port_t
    GPIO_em357_e GPIOx;
    u8_t Pin;
} __attribute__((__packed__)) em357_Port_t;

//}Structures/ Enums
//{ Defines missing in HAL-library

/* GPIO_PxCFGx Bit Field Values */
// This is a patch for the STM32W Register Mapping in the SimpleMAC library
// regs.h line 4756 ff
// Alternate Output (push-pull) Special SCLK mode
#define GPIOCFG_OUT_ALT_SPECIAL_SCLK                (0xBu)


//}
//{ Architecture-Defines (required by ttc_gpio_types.h)

#define ttc_GPIO_BANK  GPIO_em357_e
#define ttc_Port_t     em357_Port_t
#define GPIO_BANK_A    eg_GPIOA
#define GPIO_BANK_B    eg_GPIOB
#define GPIO_BANK_C    eg_GPIOC


//{ Define shortcuts for all pins of GPIO Bank A
#define PIN_PA0  GPIO_BANK_A,0
#define PIN_PA1  GPIO_BANK_A,1
#define PIN_PA2  GPIO_BANK_A,2
#define PIN_PA3  GPIO_BANK_A,3
#define PIN_PA4  GPIO_BANK_A,4
#define PIN_PA5  GPIO_BANK_A,5
#define PIN_PA6  GPIO_BANK_A,6
#define PIN_PA7  GPIO_BANK_A,7
#define PIN_PA8  GPIO_BANK_A,8
#define PIN_PA9  GPIO_BANK_A,9
#define PIN_PA10 GPIO_BANK_A,10
#define PIN_PA11 GPIO_BANK_A,11
#define PIN_PA12 GPIO_BANK_A,12
#define PIN_PA13 GPIO_BANK_A,13
#define PIN_PA14 GPIO_BANK_A,14
#define PIN_PA15 GPIO_BANK_A,15
//}BankA
//{ Define shortcuts for all pins of GPIO Bank B
#define PIN_PB0  GPIO_BANK_B,0
#define PIN_PB1  GPIO_BANK_B,1
#define PIN_PB2  GPIO_BANK_B,2
#define PIN_PB3  GPIO_BANK_B,3
#define PIN_PB4  GPIO_BANK_B,4
#define PIN_PB5  GPIO_BANK_B,5
#define PIN_PB6  GPIO_BANK_B,6
#define PIN_PB7  GPIO_BANK_B,7
#define PIN_PB8  GPIO_BANK_B,8
#define PIN_PB9  GPIO_BANK_B,9
#define PIN_PB10 GPIO_BANK_B,10
#define PIN_PB11 GPIO_BANK_B,11
#define PIN_PB12 GPIO_BANK_B,12
#define PIN_PB13 GPIO_BANK_B,13
#define PIN_PB14 GPIO_BANK_B,14
#define PIN_PB15 GPIO_BANK_B,15
//}BankB
//{ Define shortcuts for all pins of GPIO Bank C
#define PIN_PC0  GPIO_BANK_C,0
#define PIN_PC1  GPIO_BANK_C,1
#define PIN_PC2  GPIO_BANK_C,2
#define PIN_PC3  GPIO_BANK_C,3
#define PIN_PC4  GPIO_BANK_C,4
#define PIN_PC5  GPIO_BANK_C,5
#define PIN_PC6  GPIO_BANK_C,6
#define PIN_PC7  GPIO_BANK_C,7
#define PIN_PC8  GPIO_BANK_C,8
#define PIN_PC9  GPIO_BANK_C,9
#define PIN_PC10 GPIO_BANK_C,10
#define PIN_PC11 GPIO_BANK_C,11
#define PIN_PC12 GPIO_BANK_C,12
#define PIN_PC13 GPIO_BANK_C,13
#define PIN_PC14 GPIO_BANK_C,14
#define PIN_PC15 GPIO_BANK_C,15
//}BankC

//}Defines
//{ includes2

#include "ttc_gpio_types.h"  // must be included AFTER defining some ttc_XXXX thingies
//? #include "core_cm3.h"

//}includes2
//{ Prototypes

/* converts from architecture independent pin type to em357 pendant
 * @param    Type   one of tgm_analog_in, tgm_input_floating, tgm_input_pull_down, tgm_input_pull_up, tgm_output_open_drain, tgm_output_push_pull, tgm_alternate_function_push_pull, tgm_alternate_function_open_drain
 * @return   one of GPIO_Mode_AIN, GPIO_Mode_IN_FLOATING, GPIO_Mode_IPD, GPIO_Mode_IPU, GPIO_Mode_Out_OD, GPIO_Mode_Out_PP, GPIO_Mode_Out_OD, GPIO_Mode_AF_PP, GPIO_Mode_AF_OD
 */
GPIOMode_TypeDef em357_map_Type(ttc_gpio_mode_e Type);

/* converts from architecture independent pin speed to em357 pendant
 * @param    Type   one of tgs_Min, tgs_2MHz, tgs_10MHz, tgs_50MHz, tgs_Max
 * @return   one of GPIO_Speed_2MHz, GPIO_Speed_10MHz, GPIO_Speed_50MHz
 */
GPIOSpeed_TypeDef em357_map_Speed(ttc_gpio_speed_e Speed);

/* initializes single port bit for use as input/ output
 * @param Port  filled out struct as being returned by em357_loadPort_variable()
 * @param Type   one of GPIO_Mode_AIN, GPIO_Mode_IN_FLOATING, GPIO_Mode_IPD, GPIO_Mode_IPU, GPIO_Mode_Out_OD, GPIO_Mode_Out_PP, GPIO_Mode_Out_OD, GPIO_Mode_AF_PP, GPIO_Mode_AF_OD
 * @param Speed  one of GPIO_Speed_2MHz, GPIO_Speed_10MHz, GPIO_Speed_50MHz
 */
void em357_initPort_variable(em357_Port_t* Port, GPIOMode_TypeDef Type);
void em357_initPort2_variable(em357_Port_t* Port, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed);

/* initializes single port bit for use as input/ output
 * These are the real low level port bit functions with smallest overhead,
 * but they require to provide GPIOx and Pin on every function call.
 *
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIOD, GPIOE, GPIOF
 * @param Pin    0..15
 */
void em357_initPort2(GPIO_em357_e GPIOx, u8_t Pin, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed);
void em357_initPort2(GPIO_em357_e GPIOx, u8_t Pin, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed);

/* allows to load struct from list type definition
 *
 * @param Port   points to structure to be filled with given data
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIOD, GPIOE, GPIOF
 * @param Pin    0..15
 *
 * Example:
 * #define PB_LED1=GPIO_BANK_C,6
 * em357_Port_t Port;
 * em357_loadPort_variable(&Port, PB_LED1);
 * em357_initPort_variable(&Port, GPIO_Mode_Out_PP);
 * em357_portSet_variable(&Port);
 */
void em357_loadPort_variable(em357_Port_t* Port, GPIO_em357_e GPIOx, u8_t Pin);

/* read/ set to 1/ set to 0 current port pin
 * These functions provide a more dynamic and debuggable way to access port bits.
 *
 * @param Port  filled out struct as being returned by em357_loadPort_variable()
 */
BOOL em357_portGet_variable(em357_Port_t* Port);
void em357_portSet_variable(em357_Port_t* Port);
void em357_portClr_variable(em357_Port_t* Port);

/* read/ set to 1/ set to 0 current port pin
 * These are the real low level port bit functions with smallest overhead,
 * but they require to provide GPIOx and Pin on every function call.
 *
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIOD, GPIOE, GPIOF
 * @param Pin    0..15
 */
BOOL em357_portGet(GPIO_em357_e GPIOx, u8_t Pin);
void em357_portSet(GPIO_em357_e GPIOx, u8_t Pin);
void em357_portClr(GPIO_em357_e GPIOx, u8_t Pin);

//} Prototypes

#endif
