/*{ lcd_k320qvb.h *******************************

  Low-level driver for LCD Controller K320QVB as used in ARMSTM32-LCD 
  prototype board form Olimex Corp. 

  written by Gregor Rebel 2012  

}*/

#ifndef K320QVB
#define K320QVB

//{ Includes 1 ***********************************************************

#include "ttc_basic.h"

//}Includes 1
//{ Includes *************************************************************

#include "ttc_gpio.h"
#include "ttc_task.h"
#include "ttc_interrupt.h"
#include "ttc_memory.h"
#include "ttc_gfx_types.h"

#ifdef EXTENSION_100_board_olimex_lcd
  #include "lcd-320x240.h"
#endif

//}Includes
//{ externals provided by ttc_gfx.c *************************************

extern ttc_gfx_generic_t* ttc_gfx_CurrentDisplay;
extern u16_t ttc_gfx_CurrentColorFg;
extern u16_t ttc_gfx_CurrentColorBg;

//}externals
//{ Defines *************************************************************

//LCD measurements

//? #ifndef LCD_K320QVB_START_X
//? #define LCD_K320QVB_START_X 10 // ToDo: What's this for?
//? #endif
//? #ifndef LCD_K320QVB_START_Y
//? #define LCD_K320QVB_START_Y 10 // ToDo: What's this for?
//? #endif

//? #define BUTTONHEIGHT 18
//? #define BUTTONWIDTH BUTTONHEIGHT*3
//? #define MEDIUMLENGHT 10

#define LCD_CS PIN_PC8 		          //LCD Chip Select
#define LCD_RS PIN_PC9                //LCD Reset Command
#define LCD_WR PIN_PC10               //LCD Write Command Bit
#define LCD_RD PIN_PC11               //LCD Read Command Bit
#define LCD_DB00 PIN_PC0              //LCD Data Bit
#define LCD_DB01 PIN_PC1              //LCD Data Bit
#define LCD_DB02 PIN_PC2              //LCD Data Bit
#define LCD_DB03 PIN_PC3              //LCD Data Bit
#define LCD_DB04 PIN_PC4              //LCD Data Bit
#define LCD_DB05 PIN_PC5              //LCD Data Bit
#define LCD_DB06 PIN_PC6              //LCD Data Bit
#define LCD_DB07 PIN_PC7
#define LCD_DB08 PIN_PB8              //LCD Data Bit
#define LCD_DB09 PIN_PB9              //LCD Data Bit
#define LCD_DB10 PIN_PB10             //LCD Data Bit
#define LCD_DB11 PIN_PB11             //LCD Data Bit
#define LCD_DB12 PIN_PB12             //LCD Data Bit
#define LCD_DB13 PIN_PB13             //LCD Data Bit
#define LCD_DB14 PIN_PB14             //LCD Data Bit
#define LCD_DB15 PIN_PB15             //LCD Data Bit
#define LCDBUS_RSLOW_ADDR	((volatile u16_t *) 0x60000000)
#define LCDBUS_RSHIGH_ADDR	((volatile u16_t *)(0x60000000 | (1<<(19+1))))

//}Defines
//{ Function prototypes **************************************************

// initialize the LCD , white Background
void k320qvb_init();

// Reads the Reg-ID of the TFT-driver
u16_t k320qvb_read_register(u16_t Register);

/** write value into LCD-register
  *
  * @param Register   16-bit address inside K320QVB
  * @param Value      8-bit value to write into register
  */
// void k320qvb_write_register(u16_t Register, u16_t Value);
#define k320qvb_write_register(Register, Value) k320qvb_write_command(Register); *LCDBUS_RSHIGH_ADDR = (u16_t) Value

#define k320qvb_write_command(Command)  *LCDBUS_RSLOW_ADDR = (u16_t) Command


void k320qvb_clear();

/** define new color foreground/ background
  */
void k320qvb_set_color_fg8(u8_t Color);
void k320qvb_set_color_fg24(u32_t Color);
void k320qvb_set_color_bg24(u32_t Color);
void k320qvb_set_color_fg16(u32_t Color);
void k320qvb_set_color_bg16(u32_t Color);

// draw horizontal/ vertical line
void k320qvb_line_horizontal(u16_t X, u16_t Y, s16_t Length);
void k320qvb_line_vertical(u16_t X, u16_t Y, s16_t Length);

/** sets single pixel at current cursor position in given color */
// void k320qvb_put_pixel(u16_t Color);
#define k320qvb_put_pixel(Color) *LCDBUS_RSHIGH_ADDR = Color

#ifdef EXTENSION_500_ttc_font  // font attributes
void k320qvb_put_char(u8_t Character);
void k320qvb_put_char_solid(u8_t Character);
#endif

/** sets level of backlight
  *
  * @param Level   Brightness level (0 = off, 255 = max)
  */
void k320qvb_set_backlight(u8_t Level);

/** move pixel cursor to given location
  *
  * @param X  x-coordinate
  * @param X  y-coordinate
  */
// void k320qvb_set_cursor(u16_t X,u16_t Y);
#define k320qvb_set_cursor(X, Y)  k320qvb_write_register(0x20, X); k320qvb_write_register(0x21, Y); k320qvb_write_command(0x22)
#define k320qvb_set_cursor_x(X)   k320qvb_write_register(0x20, X); k320qvb_write_command(0x22)
#define k320qvb_set_cursor_y(Y)   k320qvb_write_register(0x21, Y); k320qvb_write_command(0x22)
//#define k320qvb_set_cursor_x(X)   k320qvb_write_register(0x20, X)
//#define k320qvb_set_cursor_y(Y)   k320qvb_write_register(0x21, Y)

void k320qvb_set_display_window(uint8_t Xpos, uint16_t Ypos, uint8_t Height, uint16_t Width);

void k320qvb_rect_fill(u16_t X, u16_t Y, s16_t Width, s16_t Height);

void k320qvb_get_configuration();

//}Function prototypes
//{ Macro support for ttc_gfx.h

#define _ttc_gfx_driver_set_pixel_at(X,Y)  k320qvb_set_cursor(X, Y); k320qvb_put_pixel(ttc_gfx_CurrentColorFg)
#define _ttc_gfx_driver_clr_pixel_at(X,Y)  k320qvb_set_cursor(X, Y); k320qvb_put_pixel(ttc_gfx_CurrentColorBg)
#define _ttc_gfx_driver_set_cursor(X,Y)    k320qvb_set_cursor(X, Y)

//}Macros

#endif // K320QVB
