#ifndef STM32_USART_TYPES_H
#define STM32_USART_TYPES_H

/*{ stm32_usart_types.h **********************************************************
 
                      The ToolChain
                      
   Universal Synchronous Asynchronous Receiver and Transmitters (USARTs)
   for ARM CortexM3 microcontrollers.
   Currently tested to comply to STM32F10x types.

   Architecture specific structure definitions.

   written by Gregor Rebel 2012

 
}*/
//{ Defines/ TypeDefs ****************************************************

// maximum amount of available USART/ UART units in current architecture
#define TTC_USART_MAX_AMOUNT 5

//} Defines
//{ Includes *************************************************************

#include "stm32_gpio.h"
#include "stm32_registers.h"

//} Includes *************************************************************
//{ Structures/ Enums ****************************************************


typedef struct {  // architecture specific configuration data of single USART 
  USART_t* Base;                // base address of USART registers to be used for this USART
  u8_t PhysicalIndex;           // index that directly corresponds to functional unit (0=USART1, 1=USART2, ...)
  //ttc_usart_generic_t* Generic; // architecture independent configuration data of this USART
  stm32_Port_t PortTxD;         // port pin for TxD
  stm32_Port_t PortRxD;         // port pin for RxD
  stm32_Port_t PortRTS;         // port pin for RTS
  stm32_Port_t PortCTS;         // port pin for CTS
  stm32_Port_t PortCLK;         // port pin for CLK
} __attribute__((__packed__)) stm32_usart_architecture_t;

typedef stm32_usart_architecture_t ttc_usart_architecture_t;

//} Structures/ Enums

#endif //STM32_USART_TYPES_H
