/*{ extensions.c *********************************************************
 *
 *               ToolChain_STM32
 *
 * This code manages activated source-code extensions
 * written 2010-2012 by Gregor Rebel
 *
 * Basic framework template for
 *   - Olimex STM32-P107/ STM32-H107/ STM32-LCD eval board
 *   - StdPeripheralsLibrary CMSIS V3.4.0 (tested)
 *   - GNU toolchain from CodeSourcery G++
 *   - FreeRTOS v7.0.1
 *   - FixPointLib 
 *   - uIP TCP/IP-Stack
 *   - Color LCD Panel 320x240
 *   - local/ remote debugging (JTAG-Adapter may be connected to different computer)
 *
 * Coding-style
 *  + gnu99
 *    It's year 2012, get a decent compiler if your's does not comply with this.
 *    arm-none-eabi-gcc-4.4.1 does support gnu99.
 *     
 *  + Text Folding
 *    This source makes use of text folding to improve hierarchical readability 
 *    It has been written with jEdit + plugin Configurable Fold Handler.
 *    Activate folding in your favourite text-editor for { and } 
 *    (fold-start and -end) for best visual experience.
 *
}*/
//{ Includes *************************************************************

#include "ttc_extensions.h"

//} Includes
//{ Global Variables *****************************************************

    struct FirstTaskArguments TaskArguments;

//} Global Variables
//{ Function definitions *************************************************

void startExtensions(void(*MainTask)(void*), u16_t StackSize, void* MainArgument) {
    ttc_hardware_init();
    TaskArguments.MainTask     = MainTask;
    TaskArguments.MainArgument = MainArgument;
    
    ttc_task_create(firstTask,        // function to start as thread
                    "firstTask",      // thread name (just for debugging)
                    StackSize,        // stack size
                    &TaskArguments,   // passed as argument to firstTask()
                    (Base_t) 0,       // task priority (higher values mean more process time)
                    NULL              // can return a handle to created task
                    );
    
    ttc_task_start_scheduler();
}
void firstTask(void *pvParameters) {

    struct FirstTaskArguments* Arguments=(struct FirstTaskArguments*) pvParameters;

#ifdef EXTENSION_600_example_channel
    example_channel_start();
#endif
#ifdef EXTENSION_600_example_usart
    example_usart_start();
#endif
#ifdef EXTENSION_600_example_threading_queues
    threading_start();
#endif
#ifdef EXTENSION_600_example_threading_semaphores
    threading_start();  
#endif
#ifdef EXTENSION_600_example_leds
    leds_start();
#endif
#ifdef EXTENSION_600_example_radio
    example_radio_start();
#endif
#ifdef EXTENSION_600_example_radio_serial
    example_radio_serial_start();
#endif
#ifdef EXTENSION_600_example_usb_vcp
    example_usb_vcp_start();
#endif
#ifdef EXTENSION_600_example_usb_host_hid
    example_usb_host_hid_start();
#endif
#ifdef EXTENSION_600_example_gfx_benchmark
    example_gfx_benchmark_start();
#endif
#ifdef EXTENSION_600_example_gfx_printf
    lcd_start_printf();
#endif

#ifdef EXTENSION_600_example_gfx_boxes
    lcd_start_boxes();
#endif

#if defined(EXTENSION_600_example_spi_slave) || defined(EXTENSION_600_example_spi_master)
    example_spi_start();
#endif

#if defined(EXTENSION_600_example_i2c_master) || defined(EXTENSION_600_example_i2c_slave)
    example_i2c_start();
#endif

#ifdef EXTENSION_http_server

#endif

    if (Arguments->MainTask != NULL) {
      void(*MainTask)(void*)=Arguments->MainTask;
      MainTask(Arguments->MainArgument);
    }
    while (1) mSleep(1000);
}

//} Function definitions
