#ifndef TTC_GFX
#define TTC_GFX
/*{ Template_FreeRTOS::.h ************************************************
}*/

//{ Includes *************************************************************
#include "ttc_basic.h"
#include "ttc_memory.h"
#include "ttc_string.h"
#include "ttc_gfx_types.h"
#include "gfx_image.h"
#include "math_trigonometry.h"

#ifdef EXTENSION_500_ttc_font
  #include "ttc_font.h"
#endif

#ifdef EXTENSION_400_lcd_320x240_ili9320
  #include "lcd_ili9320.h"
#endif
#ifdef EXTENSION_400_lcd_320x240_K320QVB
  #include "lcd_k320qvb.h"
#endif
#ifdef EXTENSION_500_ttc_input
  #include "ttc_input.h"
#endif

//} Includes

//{ Defines/ TypeDefs ****************************************************

//} Defines

//{ Structures/ Enums ****************************************************
//} Structures/ Enums

//{ Global Variables *****************************************************


extern void (*function_put_char)(u8_t Char);
extern void (*function_put_char_solid)(u8_t Char);
extern void (*function_clear)();
extern void (*function_line)(u16_t X1, u16_t Y1, u16_t X2, u16_t Y2);
extern void (*function_line_horizontal)(u16_t X, u16_t Y, s16_t Length);
extern void (*function_line_vertical)(u16_t X, u16_t Y, s16_t Length);
extern void (*function_rect)(u16_t X,u16_t Y, s16_t Width, s16_t Height);
extern void (*function_rect_fill)(u16_t X, u16_t Y, s16_t Width, s16_t Height);
extern void (*function_circle)(u16_t CenterX, u16_t CenterY, u16_t Radius);
extern void (*function_circle_segment)(u16_t CenterX, u16_t CenterY, u16_t Radius, u16_t StartAngle, u16_t EndAngle);
extern void (*function_set_backlight)(u8_t Level);

//} Global Variables

//{ Function prototypes **************************************************

/** returns amount of display devices available in current configuration
 * @return amount of available graphic displays
 */
u8_t ttc_gfx_get_max_display_index();

/** returns configuration of indexed display (asserts if no valid configuration was found)
 *
 * @param DisplayIndex    device index of display to init (1..ttc_display_get_max_DisplayIndex())
 * @return                configuration of indexed Display
 */
ttc_gfx_generic_t* ttc_gfx_get_configuration(u8_t DisplayIndex);

/** switch current display (if >1 displays have been configured)
 *
 * @param DisplayIndex    device index of display to init (1..ttc_display_get_max_DisplayIndex())
 */
void ttc_gfx_display(u8_t DisplayIndex);

/** draws line in current ColorFg from (X,Y) to (X+DX-1, Y+DY-1)
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 * @param X1              x-coordinate of starting point
 * @param Y1              y-coordinate of starting point
 * @param X2              x-coordinate of end point
 * @param Y2              x-coordinate of end point
 */
#define ttc_gfx_line(X1, Y1, X2, Y2) function_line(X1, Y1, X2, Y2)
// inline void ttc_gfx_line(u16_t X1, u16_t Y1, u16_t X2, u16_t Y2) { function_line(X1, Y1, X2, Y2); }
void _ttc_gfx_generic_line(u16_t X1, u16_t Y1, u16_t X2, u16_t Y2);

/** draws rectangle in current ColorFg from (X,Y) to (X+Width-1, Y+Height-1)
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 * @param X               x-coordinate of starting point
 * @param Y               y-coordinate of starting point
 * @param Width           distance to corner opposing starting point in x-direction
 * @param Height          distance to corner opposing starting point in y-direction
 */
#define ttc_gfx_rect(X, Y, Width, Height)  function_rect(X, Y, Width, Height)
// inline void ttc_gfx_rect(u16_t X, u16_t Y, s16_t Width, s16_t Height) { function_rect(X, Y, Width, Height); }
void _ttc_gfx_generic_rect(u16_t X, u16_t Y, s16_t Width, s16_t Height);

/** draws filled rectangle in current ColorFg from (X,Y) to (X+Width-1, Y+Height-1)
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 * @param X               x-coordinate of starting point
 * @param Y               y-coordinate of starting point
 * @param Width           distance to corner opposing starting point in x-direction
 * @param Height          distance to corner opposing starting point in y-direction
 */
#define ttc_gfx_rect_fill(X, Y, Width, Height)  function_rect_fill(X, Y, Width, Height)
// inline void ttc_gfx_rect_fill(u16_t X,u16_t Y, s16_t Width,s16_t Height) { function_rect_fill(X, Y, Width, Height); }
void _ttc_gfx_generic_rect_fill(u16_t X,u16_t Y, s16_t Width,s16_t Height);

/** draws circle in current ColorFg around (CenterX, CenterY) of radius Radius
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 * @param CenterX         x-coordinate of starting point
 * @param CenterY         y-coordinate of starting point
 * @param Radius          distance to corner opposing starting point in x-direction
 */
#define ttc_gfx_circle(CenterX, CenterY, Radius) function_circle(CenterX, CenterY, Radius)
// inline void ttc_gfx_circle(u16_t CenterX, u16_t CenterY, u16_t Radius) { function_circle(CenterX, CenterY, Radius); }
void _ttc_gfx_generic_circle(u16_t CenterX, u16_t CenterY, u16_t Radius);

/** draws circle segment in current ColorFg around (CenterX, CenterY) of radius Radius
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 * @param CenterX         x-coordinate of starting point
 * @param CenterY         y-coordinate of starting point
 * @param StartAngle      0..360 - circle segment is drawn counter clockwise
 * @param EndAngle        0..360 - circle segment is drawn counter clockwise
 * @param Radius          distance to corner opposing starting point in x-direction
 */
#define ttc_gfx_circle_segment(CenterX, CenterY, Radius, StartAngle, EndAngle) function_circle_segment(CenterX, CenterY, Radius, StartAngle, EndAngle)
// inline void ttc_gfx_circle_segment(u16_t CenterX, u16_t CenterY, u16_t Radius, u16_t StartAngle, u16_t EndAngle) { function_circle_segment(CenterX, CenterY, Radius, StartAngle, EndAngle); }
void _ttc_gfx_generic_circle_segment(u16_t CenterX, u16_t CenterY, u16_t Radius, u16_t StartAngle, u16_t EndAngle);

void ttc_gfx_image(u16_t X, u16_t Y, gfxImage *img);

/** initialize indexed display for use
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 */
void ttc_gfx_init(u8_t DisplayIndex);

/** set font to be used for text operations
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 * @param Font            font to use for all further text operations
 */
void ttc_gfx_set_font(const ttc_font_data_t* Font);

void ttc_gfx_printSolid(const char *string);

/** prints given string at current cursor position (transparent)
  *
  * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
  * @param String       text to print
  * @param MaxSize      >= 0: no more than this amount of characters will be printed
  */
void ttc_gfx_print(const char* String, Base_t MaxSize);

/** prints string in given memory block at current cursor position (transparent)
  *
  * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
  * @param Block       text to print
  */
void ttc_gfx_print_block(ttc_memory_block_t* Block);

/** prints string in given memory block at current cursor position (intransparent)
  *
  * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
  * @param Block       text to print
  */
void ttc_gfx_print_block_solid(ttc_memory_block_t* Block);

/** prints given string at current cursor position (not transparent)
  *
  * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
  * @param String       text to print
  * @param MaxSize      >= 0: no more than this amount of characters will be printed
  */
void ttc_gfx_print_solid(const char* String, Base_t MaxSize);

/** prints given string at current cursor position + surrounds it by a rectangle
  *
  * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
  * @param String       text to print
  * @param MaxSize      >= 0: no more than this amount of characters will be printed
  * @param InnerSpacing extra space between rectangle and text (pixels)
  */
void ttc_gfx_print_boxed(const char* String, Base_t MaxSize, u8_t InnerSpacing);

/** set display to be used for stdout functions
  *
  * @param DisplayIndex  1..ttc_gfx_get_max_DisplayIndex() index of display to use
  */
void ttc_gfx_stdout_set(u8_t DisplayIndex);

/** print all characters from given block onto stdout display
  *
  * Note: use in conjunction with ttc_string.c:ttc_string_register_stdout()
  * Note: call ttc_gfx_stdout_set() before to change to select desired display (multi-displays only)
  */
void ttc_gfx_stdout_print_block(ttc_memory_block_t* Block);

/** print all characters from given string until first zero byte onto stdout display
  *
  * Note: use in conjunction with ttc_string.c:ttc_string_register_stdout()
  * Note: call ttc_gfx_stdout_set() before to change to select desired display (multi-displays only)
  */
void ttc_gfx_stdout_print_string(const u8_t* String, Base_t MaxSize);

/** clear display with current background color
  *
  * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
  */
#define ttc_gfx_clear() function_clear();
// inline void ttc_gfx_clear() { function_clear(); }
void _ttc_gfx_generic_clear();


/** reset display
  *
  * @param DisplayIndex  1..ttc_gfx_get_max_DisplayIndex() index of display to use
  */
void ttc_gfx_reset(u8_t DisplayIndex);

/** sets level of backlight (if available)
  *
  * @param Level         Brightness level (0 = off, 255 = max)
  */
#define ttc_gfx_set_backlight(Level) function_set_backlight(Level)
// inline void ttc_gfx_set_backlight(u8_t Level) { function_set_backlight(Level); }
void _ttc_gfx_generic_set_backlight(u8_t Level);

/** Moves pixel cursor to a new location.
  *
  * Many drawing operations start at the current cursor position.
  * Some drawing operations automatically move the cursor.
  *
  * @param X   x-coordinate of new drawing position
  * @param Y   x-coordinate of new drawing position
  */
void ttc_gfx_set_cursor(u16_t X,u16_t Y);
/** little bit faster than ttc_gfx_set_cursor(); does not check if coordinates are within screen */
#define _ttc_gfx_set_cursor(X, Y)  ttc_gfx_CurrentDisplay->CursorX=X; ttc_gfx_CurrentDisplay->CursorY=Y; _ttc_gfx_driver_set_cursor(X, Y)

void ttc_gfx_set_color_bg24(u32_t Color);
void ttc_gfx_set_color_bg_palette(u8_t PaletteIndex);
void ttc_gfx_set_color_fg24(u32_t Color);
void ttc_gfx_set_color_fg_palette(u8_t PaletteIndex);
void ttc_gfx_set_palette(u16_t* Palette, u8_t Size);
void ttc_gfx_set_pixel(u16_t X, u16_t Y);

/** set foreground color to given 16-bit value
  *
  * @param Color  16-bit values are display specific use ttc_gfx_set_color_fg24() for conversion
  */
void ttc_gfx_set_color_fg16(u16_t Color);

/** set background color to given 16-bit value
  *
  * @param Color  16-bit values are display specific use ttc_gfx_set_color_fg24() for conversion
  */
void ttc_gfx_set_color_bg16(u16_t Color);

/** moves text cursor to given location
  *
  * Coordinates for text operations are calculated as follows:
  * CursorX = TextShiftX + TextX * Font->CharWidth
  * CursorY = TextShiftY + TextY * Font->CharWidth
  *
  * @param DisplayIndex  1..ttc_gfx_get_max_DisplayIndex() index of display to use
  * @param X             x-coordinate of new position
  * @param Y             y-coordinate of new position
  */
void ttc_gfx_text_cursor_set(u16_t TextX, u16_t TextY);

/** moves text cursor one step into one direction
  *
  * @param DisplayIndex  1..ttc_gfx_get_max_DisplayIndex() index of display to use
  */
void ttc_gfx_text_cursor_right();
void ttc_gfx_text_cursor_left();
void ttc_gfx_text_cursor_up();
void ttc_gfx_text_cursor_down();

/** calculates start column required to center given string on display for current font
  *
  * @param String   String to center
  * @param MaxSize  no more characters than this are used from string
  */
u16_t ttc_gfx_text_center(const char* String, Base_t MaxSize);

void ttc_gfx_missing_implementation(u16_t X, u16_t Y);

//} Function prototypes
//{ private functions

/** draws line in current ColorFg from (X, Y) to (Left+Length-1, Y)
 *
 * Note: Before calling this function, make sure that ttc_gfx_CurrentDisplay has correct value!
 *
 * @param Left            x-coordinate of starting point
 * @param Y               y-coordinate of starting point
 * @param Length          distance to end point of line
 * @return                !=NULL: pointer to struct ttc_display_generic_t
 */
void _ttc_gfx_generic_line_horizontal(u16_t X, u16_t Y, s16_t Length);

/** draws line in current ColorFg from (X, Y) to (X, Y + Length-1)
 *
 * Note: Before calling this function, make sure that ttc_gfx_CurrentDisplay has correct value!
 *
 * @param Left            x-coordinate of starting point
 * @param Y               y-coordinate of starting point
 * @param Length          distance to end point of line
 * @return                !=NULL: pointer to struct ttc_display_generic_t
 */
void _ttc_gfx_generic_line_vertical(u16_t X, u16_t Y, s16_t Length);

/** prints given character at current cursor position (transparent)
  *
  * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
  * @param Char         single ASCII character to print
  */
#define ttc_gfx_put_char(Char) function_put_char(Char)
// inline void ttc_gfx_put_char(u8_t Char) { function_put_char(Char); }
void _ttc_gfx_generic_put_char(u8_t Char);

/** prints given character at current cursor position (intransparent)
  *
  * Note: Before calling this function, make sure that ttc_gfx_CurrentDisplay has correct value!
  *
  * @param Char         single ASCII character to print
  */
#define ttc_gfx_put_char_solid(Char) function_put_char_solid(Char)
// inline void ttc_gfx_put_char_solid(u8_t Char) { function_put_char_solid(Char); }
void _ttc_gfx_generic_put_char_solid(u8_t Char);

/** loads configuration of indexed display into ttc_gfx_CurrentDisplay (asserts if no valid configuration was found)
 *
 * Note: call ttc_gfx_display() before to change to desired display (multi-displays only)
 * @return                configuration of indexed Display
 */
void _ttc_gfx_update_configuration(u8_t DisplayIndex);

//}private functions
//{ Macros

//}Macros
//{ Check availability of low-level driver macros

/** paint pixel at given position in foreground color + move cursor to the right */
#ifndef _ttc_gfx_driver_set_pixel_at // no implementation available
  #warning Missing low-level implementation!
  #define _ttc_gfx_driver_set_pixel_at(X,Y) ttc_gfx_missing_implementation(X, Y)
#endif

/** paint pixel at given position in background color + move cursor to the right */
#ifndef _ttc_gfx_driver_clr_pixel_at // no implementation available
  #warning Missing low-level implementation!
  #define _ttc_gfx_driver_clr_pixel_at(X,Y) ttc_gfx_missing_implementation(X, Y)
#endif

/** move pixel cursor to given position */
#ifndef _ttc_gfx_driver_set_cursor // no implementation available
  #warning Missing low-level implementation!
  #define _ttc_gfx_driver_set_cursor(X,Y) ttc_gfx_missing_implementation(X, Y)
#endif

//}Macros



#endif
