/*{ ttc_gpio.c ***********************************************

 Written by Gregor Rebel 2010-2012

 Architecture independent support for General Purpose Input Output (GPIO).

 Currently implemented architectures: stm32

}*/

#include "ttc_gpio.h"

/* functions defined as macros in ttc_gpio.h (faster)

bool ttc_gpio_get(GPIO_TypeDef* GPIOx, u8_t Pin);
void ttc_gpio_set(GPIO_TypeDef* GPIOx, u8_t Pin);
void ttc_gpio_clr(GPIO_TypeDef* GPIOx, u8_t Pin);
void ttc_gpio_setn(GPIO_TypeDef* GPIOx, u8_t Pin, BOOL Value);
u8_t ttc_gpio_create_index8(ttc_GPIO_BANK GPIOx, u8_t Pin);
void ttc_gpio_from_index8(u8_t PhysicalIndex, ttc_GPIO_BANK* GPIOx, u8_t* Pin);

Synonyms:

bool gget(GPIO_TypeDef* GPIOx, u8_t Pin);
void gset(GPIO_TypeDef* GPIOx, u8_t Pin);
void gclr(GPIO_TypeDef* GPIOx, u8_t Pin);
void gsetn(GPIO_TypeDef* GPIOx, u8_t Pin, BOOL Value);

*/

void ttc_gpio_init2_variable(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {

#ifdef TARGET_ARCHITECTURE_STM32
    #define TTC_INIT_PORT2
    stm32_gpio_variable(Port, GPIOx, Pin);
    stm32_gpio_init_variable(Port, Type, Speed);
#endif

#ifdef TARGET_ARCHITECTURE_EM357
    #define TTC_INIT_PORT2
    em357_loadPort_variable(Port, GPIOx, Pin);
    em357_initPort2_variable( Port, em357_map_Type(Type), em357_map_Speed(Speed) );
#endif

#ifndef TTC_INIT_PORT2
  #warning Missing low level implementation for ttc_gpio_init2_variable()
#endif
}
void ttc_gpio_init2(ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {
#ifdef TARGET_ARCHITECTURE_STM32
    #define TTC_GPIO_INIT2
    stm32_gpio_init(GPIOx, Pin, Type, Speed);
#endif

#ifdef TARGET_ARCHITECTURE_EM357
    #define TTC_GPIO_INIT2
    em357_gpio_init(GPIOx, Pin, Type, Speed);
#endif

#ifndef TTC_GPIO_INIT2
  #warning Missing low level implementation for ttc_gpio_init2_variable()
#endif

}
void ttc_gpio_init(ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type) {
    ttc_gpio_init2(GPIOx, Pin, Type, tgs_Max);
}
void _ttc_gpio_init_u16(ttc_GPIO_BANK Bank, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {
    ttc_gpio_init_u8_pin4(Bank, 0, Type, Speed);
    ttc_gpio_init_u8_pin4(Bank, 8, Type, Speed);
}
u16_t _ttc_gpio_get_u16(ttc_GPIO_BANK GPIOx) {

    // fallback implementation if no special low-level support is available
    u16_t Value = ttc_gpio_get_u8_pin2(GPIOx, 8);
    Value = Value << 8;
    Value |= ttc_gpio_get_u8_pin2(GPIOx, 0);

    return Value;
}
void _ttc_gpio_set_u16(ttc_GPIO_BANK Bank, u16_t Value) {

    // fallback implementation if no special low-level support is available
    ttc_gpio_set_u8_pin3(Bank, 0, 0x00ff & Value);
    ttc_gpio_set_u8_pin3(Bank, 8, (0xff00 & Value) >> 8);
}
void _ttc_gpio_init_u8_pin(ttc_GPIO_BANK Bank, u8_t FirstPin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {

    // fallback implementation if no special low-level support is available
    ttc_gpio_init2(Bank, FirstPin    , Type, Speed);
    ttc_gpio_init2(Bank, FirstPin + 1, Type, Speed);
    ttc_gpio_init2(Bank, FirstPin + 2, Type, Speed);
    ttc_gpio_init2(Bank, FirstPin + 3, Type, Speed);
    ttc_gpio_init2(Bank, FirstPin + 4, Type, Speed);
    ttc_gpio_init2(Bank, FirstPin + 5, Type, Speed);
    ttc_gpio_init2(Bank, FirstPin + 6, Type, Speed);
    ttc_gpio_init2(Bank, FirstPin + 7, Type, Speed);
}
u8_t _ttc_gpio_get_u8_pin(ttc_GPIO_BANK GPIOx, u8_t FirstPin) {

    // fallback implementation if no special low-level support is available
    u8_t Value = 0;
    if (ttc_gpio_get2(GPIOx, FirstPin  ))  Value  = 0x01;
    if (ttc_gpio_get2(GPIOx, FirstPin+1))  Value |= 0x02;
    if (ttc_gpio_get2(GPIOx, FirstPin+2))  Value |= 0x04;
    if (ttc_gpio_get2(GPIOx, FirstPin+3))  Value |= 0x08;
    if (ttc_gpio_get2(GPIOx, FirstPin+4))  Value |= 0x10;
    if (ttc_gpio_get2(GPIOx, FirstPin+5))  Value |= 0x20;
    if (ttc_gpio_get2(GPIOx, FirstPin+6))  Value |= 0x40;
    if (ttc_gpio_get2(GPIOx, FirstPin+7))  Value |= 0x80;

    return Value;
}
void _ttc_gpio_set_u8_pin(ttc_GPIO_BANK GPIOx, u8_t FirstPin, u8_t Value) {

    // fallback implementation if no special low-level support is available
    ttc_gpio_setn3(GPIOx, FirstPin,   Value & 0x01);
    ttc_gpio_setn3(GPIOx, FirstPin+1, Value & 0x02);
    ttc_gpio_setn3(GPIOx, FirstPin+2, Value & 0x04);
    ttc_gpio_setn3(GPIOx, FirstPin+3, Value & 0x08);
    ttc_gpio_setn3(GPIOx, FirstPin+4, Value & 0x10);
    ttc_gpio_setn3(GPIOx, FirstPin+5, Value & 0x20);
    ttc_gpio_setn3(GPIOx, FirstPin+6, Value & 0x40);
    ttc_gpio_setn3(GPIOx, FirstPin+7, Value & 0x80);
}
