#ifndef TTC_GPIO_TYPES_H
#define TTC_GPIO_TYPES_H

/*{ ttc_gpio_types.h ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Basic defines, enums and structures.
 *
 * Include this file from 
 * architecture independent header files (ttc_XXX.h) and 
 * architecture depend      header files (e.g. stm32_XXX.h)
 * 
}*/
//{ includes

// #include "ttc_basic.h"

//}includes
//{ Structures/ Enums ****************************************************

typedef enum {   // ttc_gpio_mode_e
    tgm_None,                            // 0
    tgm_analog_in,                       // Analog input to ADC
    tgm_input_floating,                  // Digital input, High-Z
    tgm_input_pull_down,                 // Digital input, resistor to GND active
    tgm_input_pull_up,                   // Digital input, resistor to VCC active
    tgm_output_open_drain,               // Digital output, can only pull to GND (to low)
    tgm_output_push_pull,                // Digital output, can pull to GND and VCC (to high and low)
    tgm_alternate_function_push_pull,    // Digital output from functional unit other than GPIO
    tgm_alternate_function_open_drain,   // Digital output from functional unit other than GPIO
    tgm_alternate_function_SPECIAL_SCLK, // Special output used by em357
    tgm_Unknown                          // Error condition
} ttc_gpio_mode_e;
typedef enum {   // ttc_gpio_speed_e
  tgs_None,
  tgs_Min,           // supported by: stm32
  tgs_2MHz,          // supported by: stm32
  tgs_10MHz,         // supported by: stm32
  tgs_50MHz,         // supported by: stm32
  tgs_Max,           // supported by: stm32
  tgs_Unknown
} ttc_gpio_speed_e;

//}Structures/ Enums
//{ includes2

#ifdef TARGET_ARCHITECTURE_STM32
  #include "stm32/stm32_gpio.h"
#endif

#ifdef TARGET_ARCHITECTURE_EM357
  #include "em357/em357_gpio.h"
#endif

// #include "ttc_basic.h"

//}includes2
//{ Defines


//} Defines
//{ Check if all required symbols are defined for current architecture ***********

// amount of gpio banks
#ifndef TTC_GPIO_MAX_AMOUNT
  #error Missing definition TTC_GPIO_MAX_AMOUNT for current architecture
/* Example (stm32)

#define GPIO_BANK_A GPIOA
#define GPIO_BANK_B GPIOB
#define GPIO_BANK_C GPIOC
#define GPIO_BANK_D GPIOD
#define GPIO_BANK_E GPIOE
#define GPIO_BANK_F GPIOF

*/
#endif

/*{ Universal GPIO pin assignment
 * 
 * The shortcuts defined here provide an architecture independent way of
 * defining GPIO bank + pin in one single statement.
 * Each defined statement expands to two comma separated values: ttc_GPIO_BANK, PinNo
 * ttc_GPIO_BANK is then resolved according to current activate cpu type.
 *
 * Example (stm32):
 * PA3 -> GPIO_BANK_A,3
 * 
 */
#define PIN_NULL NULL,0

#ifndef GPIO_BANK_A
  #error Missing GPIO bank definitions for current Architecture!
#endif
#ifndef PIN_PA0 //{
  #error Missing GPIO pin definitions for current Architecture!
/* Example (stm32)


//{ Define shortcuts for all pins of GPIO Bank A
#define PIN_PA0  GPIO_BANK_A,0
#define PIN_PA1  GPIO_BANK_A,1
#define PIN_PA2  GPIO_BANK_A,2
#define PIN_PA3  GPIO_BANK_A,3
#define PIN_PA4  GPIO_BANK_A,4
#define PIN_PA5  GPIO_BANK_A,5
#define PIN_PA6  GPIO_BANK_A,6
#define PIN_PA7  GPIO_BANK_A,7
#define PIN_PA8  GPIO_BANK_A,8
#define PIN_PA9  GPIO_BANK_A,9
#define PIN_PA10 GPIO_BANK_A,10
#define PIN_PA11 GPIO_BANK_A,11
#define PIN_PA12 GPIO_BANK_A,12
#define PIN_PA13 GPIO_BANK_A,13
#define PIN_PA14 GPIO_BANK_A,14
#define PIN_PA15 GPIO_BANK_A,15
//}BankA
//{ Define shortcuts for all pins of GPIO Bank B
#define PIN_PB0  GPIO_BANK_B,0
#define PIN_PB1  GPIO_BANK_B,1
#define PIN_PB2  GPIO_BANK_B,2
#define PIN_PB3  GPIO_BANK_B,3
#define PIN_PB4  GPIO_BANK_B,4
#define PIN_PB5  GPIO_BANK_B,5
#define PIN_PB6  GPIO_BANK_B,6
#define PIN_PB7  GPIO_BANK_B,7
#define PIN_PB8  GPIO_BANK_B,8
#define PIN_PB9  GPIO_BANK_B,9
#define PIN_PB10 GPIO_BANK_B,10
#define PIN_PB11 GPIO_BANK_B,11
#define PIN_PB12 GPIO_BANK_B,12
#define PIN_PB13 GPIO_BANK_B,13
#define PIN_PB14 GPIO_BANK_B,14
#define PIN_PB15 GPIO_BANK_B,15
//}BankB
...

*/
#endif //}
#ifndef ttc_GPIO_BANK //{

  #error Missing definition of ttc_GPIO_BANK for current Architecture!
/* Example (stm32)
 
#define ttc_GPIO_BANK  GPIO_TypeDef*

*/
#endif //}

//}Universal GPIO

#ifndef ttc_Port_t //{
  #error Missing definition of ttc_Port_t for current architecture!
  
/* Example definition of ttc_Port_t

typedef struct {
    GPIO_TypeDef* GPIOx;
    u8_t Pin;
} __attribute__((__packed__)) stm32_Port_t;

#define ttc_Port_t stm32_Port_t

*/

#endif //}

#endif //}

//{ Count amount of defined parallel ports
#ifdef TTC_PARALLEL16_5_BANK

#ifndef TTC_PARALLEL16_4_BANK
#error Missing definition of TTC_PARALLEL16_4_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL16_3_BANK
#error Missing definition of TTC_PARALLEL16_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL16_2_BANK
#error Missing definition of TTC_PARALLEL16_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL16_1_BANK
#error Missing definition of TTC_PARALLEL16_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK16_MAX 5

#else

#ifdef TTC_PARALLEL16_4_BANK

#ifndef TTC_PARALLEL16_3_BANK
#error Missing definition of TTC_PARALLEL16_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL16_2_BANK
#error Missing definition of TTC_PARALLEL16_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL16_1_BANK
#error Missing definition of TTC_PARALLEL16_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK16_MAX 4

#else

#ifdef TTC_PARALLEL16_3_BANK

#ifndef TTC_PARALLEL16_2_BANK
#error Missing definition of TTC_PARALLEL16_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL16_1_BANK
#error Missing definition of TTC_PARALLEL16_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK16_MAX 3

#else

#ifdef TTC_PARALLEL16_2_BANK

#ifndef TTC_PARALLEL16_1_BANK
#error Missing definition of TTC_PARALLEL16_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK16_MAX 2

#else

#ifdef TTC_PARALLEL16_1_BANK
#define TTC_GPIO_BANK16_MAX 2
#else
#define TTC_GPIO_BANK16_MAX 0
#endif

#endif

#endif

#endif


#endif

#ifdef TTC_PARALLEL08_10_BANK

#ifndef TTC_PARALLEL08_9_BANK
#error Missing definition of TTC_PARALLEL08_9_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_8_BANK
#error Missing definition of TTC_PARALLEL08_8_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_7_BANK
#error Missing definition of TTC_PARALLEL08_7_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_6_BANK
#error Missing definition of TTC_PARALLEL08_6_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_5_BANK
#error Missing definition of TTC_PARALLEL08_5_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_4_BANK
#error Missing definition of TTC_PARALLEL08_4_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_3_BANK
#error Missing definition of TTC_PARALLEL08_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 10

#else

#ifdef TTC_PARALLEL08_9_BANK

#ifndef TTC_PARALLEL08_8_BANK
#error Missing definition of TTC_PARALLEL08_8_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_7_BANK
#error Missing definition of TTC_PARALLEL08_7_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_6_BANK
#error Missing definition of TTC_PARALLEL08_6_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_5_BANK
#error Missing definition of TTC_PARALLEL08_5_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_4_BANK
#error Missing definition of TTC_PARALLEL08_4_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_3_BANK
#error Missing definition of TTC_PARALLEL08_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 9

#else

#ifdef TTC_PARALLEL08_8_BANK

#ifndef TTC_PARALLEL08_7_BANK
#error Missing definition of TTC_PARALLEL08_7_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_6_BANK
#error Missing definition of TTC_PARALLEL08_6_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_5_BANK
#error Missing definition of TTC_PARALLEL08_5_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_4_BANK
#error Missing definition of TTC_PARALLEL08_4_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_3_BANK
#error Missing definition of TTC_PARALLEL08_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 8

#else

#ifdef TTC_PARALLEL08_7_BANK

#ifndef TTC_PARALLEL08_6_BANK
#error Missing definition of TTC_PARALLEL08_6_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_5_BANK
#error Missing definition of TTC_PARALLEL08_5_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_4_BANK
#error Missing definition of TTC_PARALLEL08_4_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_3_BANK
#error Missing definition of TTC_PARALLEL08_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 7

#else

#ifdef TTC_PARALLEL08_6_BANK

#ifndef TTC_PARALLEL08_5_BANK
#error Missing definition of TTC_PARALLEL08_5_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_4_BANK
#error Missing definition of TTC_PARALLEL08_4_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_3_BANK
#error Missing definition of TTC_PARALLEL08_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 6

#else

#ifdef TTC_PARALLEL08_5_BANK

#ifndef TTC_PARALLEL08_4_BANK
#error Missing definition of TTC_PARALLEL08_4_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_3_BANK
#error Missing definition of TTC_PARALLEL08_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 5

#else

#ifdef TTC_PARALLEL08_4_BANK

#ifndef TTC_PARALLEL08_3_BANK
#error Missing definition of TTC_PARALLEL08_3_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 4

#else

#ifdef TTC_PARALLEL08_3_BANK

#ifndef TTC_PARALLEL08_2_BANK
#error Missing definition of TTC_PARALLEL08_2_BANK (parallel ports must be defined consecutive)
#endif
#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 3

#else

#ifdef TTC_PARALLEL08_2_BANK

#ifndef TTC_PARALLEL08_1_BANK
#error Missing definition of TTC_PARALLEL08_1_BANK (parallel ports must be defined consecutive)
#endif
#define TTC_GPIO_BANK08_MAX 2

#else

#ifdef TTC_PARALLEL08_1_BANK
#define TTC_GPIO_BANK08_MAX 1
#else
#define TTC_GPIO_BANK08_MAX 0
#endif

#endif

#endif

#endif

#endif

#endif

#endif

#endif

#endif

#endif

//}Count amount of defined parallel ports
//{ Prepare defined ports

#ifdef TTC_PARALLEL08_10_BANK
#ifndef TTC_PARALLE08_10_FIRSTPIN
  #define TTC_PARALLE08_10_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_10 TTC_PARALLEL08_10_BANK,TTC_PARALLE08_10_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_9_BANK
#ifndef TTC_PARALLE08_9_FIRSTPIN
  #define TTC_PARALLE08_9_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_9 TTC_PARALLEL08_9_BANK,TTC_PARALLE08_9_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_8_BANK
#ifndef TTC_PARALLE08_8_FIRSTPIN
  #define TTC_PARALLE08_8_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_8 TTC_PARALLEL08_8_BANK,TTC_PARALLE08_8_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_7_BANK
#ifndef TTC_PARALLE08_7_FIRSTPIN
  #define TTC_PARALLE08_7_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_7 TTC_PARALLEL08_7_BANK,TTC_PARALLE08_7_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_6_BANK
#ifndef TTC_PARALLE08_6_FIRSTPIN
  #define TTC_PARALLE08_6_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_6 TTC_PARALLEL08_6_BANK,TTC_PARALLE08_6_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_5_BANK
#ifndef TTC_PARALLE08_5_FIRSTPIN
  #define TTC_PARALLE08_5_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_5 TTC_PARALLEL08_5_BANK,TTC_PARALLE08_5_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_4_BANK
#ifndef TTC_PARALLE08_4_FIRSTPIN
  #define TTC_PARALLE08_4_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_4 TTC_PARALLEL08_4_BANK,TTC_PARALLE08_4_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_3_BANK
#ifndef TTC_PARALLE08_3_FIRSTPIN
  #define TTC_PARALLE08_3_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_3 TTC_PARALLEL08_3_BANK,TTC_PARALLE08_3_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_2_BANK
#ifndef TTC_PARALLE08_2_FIRSTPIN
  #define TTC_PARALLE08_2_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_2 TTC_PARALLEL08_2_BANK,TTC_PARALLE08_2_FIRSTPIN
#endif

#ifdef TTC_PARALLEL08_1_BANK
#ifndef TTC_PARALLE08_1_FIRSTPIN
  #define TTC_PARALLE08_1_FIRSTPIN 0
#endif
#define TTC_PARALLEL08_1 TTC_PARALLEL08_1_BANK,TTC_PARALLE08_1_FIRSTPIN
#endif

//}Prepare defined ports
