
#ifndef TTC_INTERRUPT_H
#define TTC_INTERRUPT_H

/*{ ttc_interrupt.h **********************************************************

                      The ToolChain

   Device independent support for interrupt requests

   Currently implemented architectures: stm32

   written by Gregor Rebel 2012


   How it works

   Different architecture drivers for usart, spi and others may define certain
   constants to add entries to interrupt definition structures.
   See ttc_interrupt_types.h for a list of valid constants.

}*/
//{ Defines/ TypeDefs ****************************************************


//} Defines
/* Defines/ TypeDefs/ structures being set by architecture driver
 *
 * Constant defines
 *
 * Structures
 *
 */
//{ Includes *************************************************************

#include "ttc_basic.h"
#include "ttc_gpio.h"

#include "ttc_interrupt_types.h"

// supported architectures
#ifdef TARGET_ARCHITECTURE_STM32
  #include "stm32/stm32_interrupt.h"
#endif

//} Includes
//{ Structures/ Enums ****************************************************

//} Structures/ Enums
//{ Global Variables *****************************************************

//} Global Variables
//{ Function prototypes **************************************************

/** initializes interrupt for given source to call given ISR
 * Note: This will not activate your interrupt. Call ttc_interrupt_usart_enable() afterwards to enable/ disable this interrupt.
 *
 * @param Source        interrupt source to use
 * @param PhysicalIndex real physical index of functional unit (e.g. 0 = USART1, 1 = USART2, ...)
 * @param ISR           function pointer of interrupt service routine to call;
 *                      ISR() gets two params: u8_t Index, void* Reference
 *                      Index     index functional unit in microcontroller (e.g. 0=first USART)
 *                      Reference hardware dependent data used by low level driver (e.g. register base address)
 * @param Argument      will be passed as argument to ISR()
 * @param OldISR        != NULL: previous stored pointer will be saved into given address
 * @param OldArgument   != NULL: previous stored argument will be saved into given address
 */
ttc_interrupt_errorcode_e ttc_interrupt_init(ttc_interrupt_type_e Source, u8_t PhysicalIndex, void (*ISR)(u8_t, void*), void* Argument, void (**OldISR)(physical_index_t, void*), void** OldArgument);

/** un-initialize given interrupt source
 * @param Source     interrupt source to use
 * @param PhysicalIndex  real physical index of functional unit (e.g. 0 = USART1, 1 = USART2, ...)
 */
ttc_interrupt_errorcode_e ttc_interrupt_deinit(ttc_interrupt_type_e Source, u8_t PhysicalIndex);

/** activates/ deactivates a previously initialized interrupt
 * @param Source        interrupt source to use
 * @param PhysicalIndex real physical index of functional unit (e.g. 0 = USART1, 1 = USART2, ...)
 * @param EnableIRQ     =0: disable interrupt; !=0: enable interrupt
 */
void ttc_interrupt_enable(ttc_interrupt_type_e Source, u8_t PhysicalIndex, BOOL EnableIRQ);

//} Function prototypes

#endif //TTC_INTERRUPT_H
