/*{ ttc_memory.c ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Scheduler and architecture independent multimemorying support.
 * 
}*/
#include "ttc_memory.h"

#if TTC_MEMORY_ENUMERATE == 1
Base_t ttc_memory_CountAlloc = 0;
#endif

              void* ttc_memory_alloc(Base_t Size) {

    Base_t* Address = NULL;

#if TTC_MEMORY_ENUMERATE == 1
    Size += sizeof(Base_t);
#endif

#ifdef EXTENSION_300_scheduler_free_rtos
    Address = (Base_t*) pvPortMalloc(Size); // provided by heap_1.c, heap_2.c or heap_3.c
#endif

    Assert(Address != NULL, ec_OutOfMemory); // could not allocate memory!

#if TTC_MEMORY_ENUMERATE == 1
    *Address = ++ttc_memory_CountAlloc; // each memory block gets its own number at Address - sizeof(Base_t)
    Address++;
#endif

    return (void*) Address;
}
              void* ttc_memory_alloc_zeroed(Base_t Size) {

    void* Address = ttc_memory_alloc(Size);
    ttc_memory_set(Address, 0, Size);

    return Address;
}
               void ttc_memory_copy(void* Destination, const void* Source, Base_t AmountBytes) {
    Assert(Destination, ec_NULL);
    Assert(Source,      ec_NULL);

#if (TARGET_DATA_WIDTH == 32) // make use of 32-bit registers to speed up copying
    while (AmountBytes > 3) {
        *((u32_t*) Destination) = *((u32_t*) Source);
        Destination = (void*) (((u32_t) Destination) + 4);
        Source      = (void*) (((u32_t) Source)      + 4);
        AmountBytes -= 4;
    }
#else
#if (TARGET_DATA_WIDTH == 16) // make use of 16-bit registers to speed up copying
    while (AmountBytes > 1) {
        *((u16_t*) Destination) = *((u16_t*) Source);
        Destination = (void*) (((u32_t) Destination) + 2);
        Source      = (void*) (((u32_t) Source)      + 2);
        AmountBytes -= 2;
    }
#endif
#endif

    while (AmountBytes-- > 0) {
        *((u8_t*) Destination) = *((u8_t*) Source);
        Destination = (void*) (((u32_t) Destination) + 1);
        Source      = (void*) (((u32_t) Source)      + 1);
    }
}
               void ttc_memory_set(void* Destination, Base_t Fill, Base_t AmountBytes) {
    Assert(Destination, ec_NULL);

#if (TARGET_DATA_WIDTH == 32) // make use of 32-bit registers to speed up copying
    if (AmountBytes > 11) {
        while (AmountBytes > 3) {
            *((u32_t*) Destination) = Fill;
            Destination = (void*) (((u32_t) Destination) + 4);
            AmountBytes -= 4;
        }
    }
#else
#if (TARGET_DATA_WIDTH == 16) // make use of 16-bit registers to speed up copying

    if (AmountBytes > 5) {
        while (AmountBytes > 1) {
            *((u16_t*) Destination) = (u16_t) Fill);
            Destination = (void*) (((u32_t) Destination) + 2);
            AmountBytes -= 2;

        }
    }

#endif
#endif

    while (AmountBytes-- > 0) {
        *((u8_t*) Destination) = (u8_t) Fill;
        Destination = (void*) (((u32_t) Destination) + 1);
    }
}
             Base_t ttc_memory_compare(Base_t A, Base_t B, Base_t AmountBytes) {
    Base_t Index = 0;

#if (TARGET_DATA_WIDTH == 32) // make use of 32-bit registers to speed up copying
    if (AmountBytes > 11) {
        while (AmountBytes > 3) {
            if (*((u32_t*) A) != *((u32_t*) B) )  // words differ: will determine exact position later
                break;
            Index += 4;
            A += 4;
            B += 4;
            AmountBytes -= 4;
        }
    }
#endif
#if (TARGET_DATA_WIDTH == 16) // make use of 16-bit registers to speed up copying
    if (AmountBytes > 11) {
        while (AmountBytes > 3) {
            if (*((u32_t*) A) != *((u32_t*) B) ) // words differ: will determine exact position later
                break;
            Index += 2;
            A += 2;
            B += 2;
            AmountBytes -= 2;
        }
    }
#endif

    while (AmountBytes-- > 0) {
        if (*((u8_t*) A) != *((u8_t*) B) )
            return Index;
        A++;
        B++;
        Index++;
    }

    return 0;  // all checked bytes are the same
}
             void*  ttc_free(void* Block) {

#ifdef EXTENSION_300_scheduler_free_rtos
    vPortFree(Block); // provided by heap_1.c, heap_2.c or heap_3.c
    return NULL;
#endif

    return Block;
}
             Base_t ttc_memory_get_free_size() {

#ifdef EXTENSION_300_scheduler_free_rtos
    return (Base_t) xPortGetFreeHeapSize();
#endif

    return 0;
}
ttc_memory_block_t* ttc_memory_alloc_block(Base_t Size, u8_t Hint, void (*releaseBuffer)(ttc_memory_block_t* Block)) {
    ttc_memory_block_t* MemoryBlock = ttc_memory_alloc(sizeof(ttc_memory_block_t) + Size);

    MemoryBlock->Size          = 0;
    MemoryBlock->MaxSize       = Size;
    MemoryBlock->UseCount      = 1;
    MemoryBlock->Hint          = Hint;
    MemoryBlock->releaseBuffer = releaseBuffer;
    MemoryBlock->Buffer        = ( (u8_t*) MemoryBlock ) + sizeof(ttc_memory_block_t);

    return MemoryBlock;
}
ttc_memory_block_t* ttc_memory_alloc_virtual_block(u8_t* Buffer, u8_t Hint, void (*releaseBuffer)(ttc_memory_block_t* Block)) {
    Assert(Buffer != NULL, ec_InvalidArgument);
    ttc_memory_block_t* MemoryBlock = ttc_memory_alloc(sizeof(ttc_memory_block_t));

    MemoryBlock->Size          = 0;
    MemoryBlock->Hint          = Hint;
    MemoryBlock->releaseBuffer = releaseBuffer;
    MemoryBlock->Buffer        = Buffer;

    return MemoryBlock;
}
               void ttc_memory_block_use(ttc_memory_block_t* Block) {
  Assert(Block->UseCount < 255, ec_UNKNOWN); // maybe endless loop calling ttc_memory_block_use() ?
  Block->UseCount++;
}
               void ttc_memory_block_release(ttc_memory_block_t* Block) {
    Assert(Block, ec_InvalidArgument);
    Assert(Block->releaseBuffer, ec_UNKNOWN);  // no release function stored!

    Block->UseCount--;
    Assert(Block->UseCount != 255, ec_InvalidImplementation); // maybe you forgot to call ttc_memory_block_use() before reusing a release memory block?
    if (Block->UseCount == 0)
        Block->releaseBuffer(Block);
}
               BOOL ttc_memory_is_constant(const void* Address) {
    (void) Address; // avoids warning "unused variable"

#ifdef TARGET_ARCHITECTURE_STM32
    if (  ( ((u32_t) Address) > 0x08000000 ) && ( ((u32_t) Address) < 0x08040000 )  )
        return TRUE;
#endif
    return FALSE;
}
               BOOL ttc_memory_is_writable(void* Address) {
#ifdef TARGET_ARCHITECTURE_STM32
    if (  ( ((u32_t) Address) > 0x20000000) && ( ((u32_t) Address) < 0x20010000)  )
        return TRUE;
#endif
    return FALSE;
}
               u8_t ttc_memory_checkpointer(void* Pointer) {
    if (!Pointer)
        Assert_Halt_NULL();

    return 0;
}
               u8_t ttc_memory_checkpointer_within(void* Pointer, void* Min, void* Max) {
    Assert( (Pointer < Min) || (Pointer > Max), ec_IllegalPointer);
    return 0;
}
 ttc_memory_pool_t* ttc_memory_pool_create(TTC_MEMORY_BLOCK_BASE BufferSize, u8_t MaxBlocks, void (*releaseBuffer)(ttc_memory_block_t* Block)) {
  Assert(releaseBuffer, ec_InvalidArgument);

  ttc_memory_pool_t* NewPool = ttc_memory_alloc_zeroed( sizeof(ttc_memory_pool_t) );
  if (NewPool) {
      NewPool->BufferSize    = BufferSize;
      NewPool->MaxBlocks     = MaxBlocks;
      NewPool->releaseBuffer = releaseBuffer;
  }
  return NewPool;
}
ttc_memory_block_t* ttc_memory_pool_get_block(ttc_memory_pool_t* Pool) {
    Assert(Pool, ec_InvalidArgument);

    ttc_task_begin_criticalsection(ttc_memory_pool_get_block); // prevent other tasks from accessing memory pools

    // Strategy of ttc_memory_pool_get_block()
    // Problem: From here on, an interrupt service routine can interrupt us.
    // Pool->ActiveList = -<N> indicates a read access of EmptyBlocks<N>
    // 1) try to get block from start of EmptyBlocks2
    // 2) try to get block from start of EmptyBlocks1
    // 3) try to allocate new block

    Assert(Pool->ActiveList, ec_InvalidImplementation); // list occupied by another task: protect access by external mutex!

    Pool->ActiveList = -2;             // trying to get block from EmptyBlocks2
    ttc_memory_block_t* EmptyBlock = Pool->EmptyBlocks2;
    if (EmptyBlock) { // got empty block: remove it from EmptyBlocks2
        Pool->EmptyBlocks2 = EmptyBlock->Next;
        if (!Pool->EmptyBlocks2)
            Pool->EmptyBlocks2_End = NULL; // list #2 is empty

    }
    else {            // got no empty block: try to get block from EmptyBlocks1

        Pool->ActiveList = -1; // ISR will now use EmptyBlocks1

        EmptyBlock = Pool->EmptyBlocks1;
        if (EmptyBlock) {
            Pool->EmptyBlocks1 = EmptyBlock->Next;
            if (!Pool->EmptyBlocks1)
                Pool->EmptyBlocks1_End = NULL; // list #1 is empty
        }
    }

    if (!EmptyBlock) { // got no empty block from either list: try to allocate new block
        Pool->ActiveList = 3; // inform ISR of a block allocation
        if ( (Pool->MaxBlocks==0) ||  // amount of blocks unlimited
             (Pool->AmountBlocks + Pool->AmountBlocks_ISR < Pool->MaxBlocks)
        ) {
            EmptyBlock = ttc_memory_alloc_block(Pool->BufferSize, 0, Pool->releaseBuffer);
            if (EmptyBlock)
                Pool->AmountBlocks++;
        }
    }

    Pool->ActiveList = 0; // unlock list
    ttc_task_end_criticalsection();

    EmptyBlock->UseCount = 1;
    return EmptyBlock;
}
ttc_memory_block_t* ttc_memory_pool_get_block_isr(ttc_memory_pool_t* Pool) {
    Assert(Pool, ec_InvalidArgument);

    ttc_memory_block_t* EmptyBlock = NULL;
    u8_t ActiveList = Pool->ActiveList;
    if ((0x7f & ActiveList) != 1) { // no task is using EmptyBlocks1: try to get empty block from it
        EmptyBlock = Pool->EmptyBlocks1;
        if (EmptyBlock) { // got block: remove it from start of list
            Pool->EmptyBlocks1 = Pool->EmptyBlocks1->Next;
            return EmptyBlock;
        }
    }
    else {
        if (ActiveList ==  1) { // task appends to end of EmptyBlocks1: try to get block from begin of EmptyBlocks1
            if (Pool->EmptyBlocks1 != Pool->EmptyBlocks1_End) { // list has >=2 entries: get first entry
                EmptyBlock         = Pool->EmptyBlocks1;
                if (EmptyBlock) {
                    Pool->EmptyBlocks1 = EmptyBlock->Next;
                    return EmptyBlock;
                }
            }
        }
        if (ActiveList == -1) { // task removes from start of EmptyBlocks1: try to get block from end of EmptyBlocks1
            if (Pool->EmptyBlocks1 != Pool->EmptyBlocks1_End) { // list has >=2 entries: get second entry
                EmptyBlock = Pool->EmptyBlocks1->Next;
                if (EmptyBlock) {
                    if (Pool->EmptyBlocks1_End == EmptyBlock) // got last block: corrent end-pointer
                        Pool->EmptyBlocks1_End = Pool->EmptyBlocks1;
                    Pool->EmptyBlocks1->Next = EmptyBlock->Next;
                    return EmptyBlock;
                }
            }
        }
    }

    // got no block from EmptyBlocks1: try EmptyBlocks2
    if ((0x7f & ActiveList) != 2) { // no task is using EmptyBlocks2: try to get empty block from it
        EmptyBlock = Pool->EmptyBlocks2;
        if (EmptyBlock) { // got block: remove it from start of list
            Pool->EmptyBlocks2 = Pool->EmptyBlocks2->Next;
            return EmptyBlock;
        }
    }

    // got no block from EmptyBlocks1 or EmptyBlocks2: try to allocate one
    if ( (Pool->MaxBlocks==0) ||  // amount of blocks unlimited
         ( (Pool->ActiveList != 3) &&
           (Pool->AmountBlocks + Pool->AmountBlocks_ISR < Pool->MaxBlocks)
         ) ||
         ( (Pool->ActiveList == 3) && // task is currently allocating a block: take this into account
           (Pool->AmountBlocks + Pool->AmountBlocks_ISR + 1 < Pool->MaxBlocks)
         )
       ) {
        EmptyBlock = ttc_memory_alloc_block(Pool->BufferSize, 0, Pool->releaseBuffer);
        if (EmptyBlock)
            Pool->AmountBlocks_ISR++;
    }

    return EmptyBlock;
}
               void ttc_memory_pool_release_block(ttc_memory_pool_t* Pool, ttc_memory_block_t* Block) {
    Block->Next = NULL;
    ttc_task_begin_criticalsection(ttc_memory_pool_release_block); // prevent other tasks from accessing memory pools

    // Strategy of ttc_memory_pool_release_block()
    // Problem: From here on, an interrupt service routine can interrupt us.
    // Pool->ActiveList = +<N> indicates a write access of EmptyBlocks<N>
    // 1) add block to end of EmptyBlocks1

    Pool->ActiveList = 1;             // tasks always append to end of EmptyBlocks1
    if (!Pool->EmptyBlocks1) {        // list is empty: insert Block
        Pool->EmptyBlocks1     = Block;
        Pool->EmptyBlocks1_End = Block;
        Block = NULL;
    }
    else {                            // list not empty: append at end of list
        Assert(Pool->EmptyBlocks1_End, ec_InvalidConfiguration); // This pointer must not be NULL
        Pool->EmptyBlocks1_End->Next = Block;
        Pool->EmptyBlocks1_End       = Block;
    }
    Pool->ActiveList = 0; // unlock list
    ttc_task_end_criticalsection();
}
               void ttc_memory_pool_release_block_isr(ttc_memory_pool_t* Pool, ttc_memory_block_t* Block) {
    Block->Next = NULL;

    // Strategy of ttc_memory_pool_release_block_isr()
    // Problem: A task may have started ttc_memory_pool_release_block() or ttc_memory_pool_get_block()
    // 1) EmptyBlocks2 not accessed by task: add block to end of EmptyBlocks2
    // 1) EmptyBlocks2     accessed by task: add block to end of EmptyBlocks1

    if ( (0x7f & Pool->ActiveList) != 2 ) { // no task accesses EmptyBlocks2: add to this list
        if (!Pool->EmptyBlocks2) {        // list is empty: insert Block
            Pool->EmptyBlocks2     = Block;
            Pool->EmptyBlocks2_End = Block;
            Block = NULL;
        }
        else {                            // list not empty: append at end of list
            Assert(Pool->EmptyBlocks2_End, ec_InvalidConfiguration); // This pointer must not be NULL
            Pool->EmptyBlocks2_End->Next = Block;
            Pool->EmptyBlocks2_End       = Block;
        }
    }
    else {                                // EmptyBlocks2 is being accessed by task: add to EmptyBlocks1
        if (!Pool->EmptyBlocks1) {        // list is empty: insert Block
            Pool->EmptyBlocks1     = Block;
            Pool->EmptyBlocks1_End = Block;
            Block = NULL;
        }
        else {                            // list not empty: append at end of list
            Assert(Pool->EmptyBlocks1_End, ec_InvalidConfiguration); // This pointer must not be NULL
            Pool->EmptyBlocks1_End->Next = Block;
            Pool->EmptyBlocks1_End       = Block;
        }
    }
}
               void* _ttc_array_ref(Base_t* Size, Base_t Index) {
     Assert(Index < *Size, ec_ArrayIndexOutOfBound);
     return Size+1; // return pointer to first array element
 }
              Base_t _ttc_array_index(Base_t* Size, Base_t Index) {
     Assert(Index < *Size, ec_ArrayIndexOutOfBound);
     return Index;
 }
