/*{ ttc_network.c ***************************************************

 This source is published under the GNU LESSER GENERAL PUBLIC LICENSE (LGPL).
 See file LEGAL.txt in this software package for details.

 Authors: <Gregor Rebel>

 Description: Basic universal network support.

}*/

#include "ttc_network.h"

//{ global variables ************************************************

//}global variables
//{ function definitions ********************************************

ttc_network_packet_t* ttc_network_alloc_packet(u16_t Size_Payload, u8_t Size_Header, u8_t Size_Footer, u8_t Hint, void (*releaseBuffer)(ttc_network_packet_t* Packet)) {
    ttc_network_packet_t* Packet = (ttc_network_packet_t*) ttc_memory_alloc(Size_Payload + Size_Header + Size_Footer + sizeof(ttc_network_packet_t));

    // calculate start of Payload[] right after Header[]
    Packet->Payload = ( (u8_t*) Packet ) + sizeof(ttc_network_packet_t) + Size_Header;

    // store release function
    Packet->releaseBuffer = releaseBuffer;

    ttc_network_reset_packet(Packet);

    return Packet;
}
                 void ttc_network_reset_packet(ttc_network_packet_t* Packet) {
    Assert_Network(Packet, ec_InvalidArgument);

    // at the beginning, all pointers point to start of Payload[]
    Packet->Header  = Packet->Payload;
    Packet->Footer  = Packet->Payload;

    // packet is empty
    Packet->PayloadSize = 0;
}
                 void ttc_network_release_packet(ttc_network_packet_t* Packet) {
    Assert(Packet, ec_InvalidArgument);
    Assert(Packet->releaseBuffer, ec_UNKNOWN);  // no release function stored!

    Packet->releaseBuffer(Packet);
}
                 void ttc_network_add_header(ttc_network_packet_t* Packet, u8_t Size) {
    Assert_Network(Packet, ec_InvalidArgument);
    Packet->Header -= Size;
    Assert_Network(Packet->Header > ((u8_t*) Packet) + sizeof(ttc_network_packet_t), ec_InvalidConfiguration); // header exceeded predefined size!
}
                 void ttc_network_add_footer(ttc_network_packet_t* Packet, u8_t Size) {
    Assert_Network(Packet, ec_InvalidArgument);
    Packet->Footer += Size;
    //X Assert_Network(Packet->Footer < ((u8_t*) Packet) + sizeof(ttc_network_packet_t) + PacketMaxSize, ec_InvalidConfiguration); // footer exceeded predefined size!
}
                 void ttc_network_del_footer(ttc_network_packet_t* Packet, u8_t Size) {
    Assert_Network(Packet, ec_InvalidArgument);
    Packet->Footer -= Size;
    Assert_Network(Packet->Footer >= (Packet->Payload + Packet->PayloadSize), ec_InvalidConfiguration); // footer moved into payload
}
                 void ttc_network_del_header(ttc_network_packet_t* Packet, u8_t Size) {
    Assert_Network(Packet, ec_InvalidArgument);
    Packet->Header += Size;
    Assert_Network(Packet->Header <= Packet->Payload, ec_InvalidConfiguration); // header-pointer moved into payload!
}

//}function definitions
