/*{ Template_FreeRTOS::.c ************************************************
 
  lcd_start_printf() spawns thread taskBrightness()

  taskBrightness()
      Defines brightness of led as Ton / (Ton + Toff)
      spawns thread taskLEDs()
      while (1) {
        while (Ton < 100) { Ton++; Toff--; }
        while (Ton >   0) { Ton--; Toff++; }
      }

  taskLEDs()
      while (1) {
        switch on LED
        wait for Ton
        switch off LED
        wait for Toff
      }

}*/

#include "example_gfx_printf.h"

// read-only configuration of current display (width, height, ...)
ttc_gfx_generic_t* DisplayConfig = NULL;

//{ Function definitions *************************************************


void lcd_printf_init() {

    u8_t MaxDisplay = ttc_gfx_get_max_display_index();
    Assert(MaxDisplay > 0, ec_DeviceNotFound); // no displays defined by board makefile!
    DisplayConfig = ttc_gfx_get_configuration(1);
    Assert(DisplayConfig, ec_DeviceNotFound);

    // set color to use for first screen clear
    ttc_gfx_set_color_bg24(GFX_COLOR24_AQUA);
    ttc_gfx_init(1);

    // initialize gfx for use with printf()
    ttc_gfx_stdout_set(1);
    ttc_string_register_stdout(ttc_gfx_stdout_print_block, ttc_gfx_stdout_print_string);

}
void lcd_start_printf() {
    lcd_printf_init();

    ttc_task_create(task_LCD,       // function to start as thread
                    "LCD",              // thread name (just for debugging)
                    512,                // stack size
                    (void *) NULL,      // passed as argument to taskLEDs()
                    1,                  // task priority (higher values mean more process time)
                    NULL                // can return a handle to created task
                    );
}
void task_LCD(void *pvParameters) {
    pvParameters = pvParameters; // avoids warning: unused parameter 'pvParameters'
    
    const char* Hallo = "Hallo";
    const char* Welt  = "Welt";

    ttc_gfx_set_color_fg24(GFX_COLOR24_NAVY);
    ttc_gfx_text_cursor_set(1, 1);
    ttc_gfx_print_boxed(Hallo, -1, 1);
    ttc_gfx_print_boxed(Welt,  -1, 1);
    ttc_gfx_text_cursor_set(1, 2);
    ttc_gfx_print("Seconds: ", -1);
    u16_t SecsX = DisplayConfig->TextX;
    u16_t SecsY = DisplayConfig->TextY;
    ttc_gfx_rect(1,1,DisplayConfig->Width-3, DisplayConfig->Height-3);
    Base_t BytesFree = ttc_memory_get_free_size();
    ttc_gfx_text_cursor_set(0, DisplayConfig->TextRows - 1);
    printf("free: %iB", BytesFree);


    u16_t Counter = 0;
    u8_t LightLevel = 255;
    while (1) {
        ttc_gfx_text_cursor_set(SecsX, SecsY);
        printf("%i", Counter++);
        ttc_gfx_set_backlight(LightLevel);
        //LightLevel = 255 - LightLevel;

        mSleep(1000);
    }
}

//} Function definitions
