#ifndef EXAMPLE_LEDS
#define EXAMPLE_LEDS

/*{ EsampleLeds::.h ************************************************
 
 Empty template for new header files.
 Copy and adapt to your needs.
 
}*/
//{ Includes *************************************************************

// Basic set of helper functions
#include "ttc_basic.h"
#include "ttc_gpio.h"
#include "ttc_task.h"
#include "ttc_watchdog.h"
#include "ttc_interrupt.h"

//} Includes
//{ Defines/ TypeDefs ****************************************************

#ifndef TTC_LED1
  #error TTC_LED1 not defined!

  // Example header-definition:
  // #define TTC_LED2   PIN_PC6    // status led 1 on Olimex proto board P107
  //
  // Example makefile-definition:
  // COMPILE_OPTS += -DTTC_LED2=PIN_PC6
#endif

//} Defines
//{ Structures/ Enums ****************************************************

typedef struct { // ConfigLED_t
  ttc_Port_t Port;       // port where LED is connected
  u8_t Brightness;       // 0 = off, 99 = 100% on
  bool Invert;           // == TRUE: invert brightness
} ConfigLED_t;

//} Structures/ Enums
//{ Function prototypes **************************************************

// basic hardware intialization
void leds_initPorts();

// main entry: initializes hardware + spawns all threads
// returns immediately
void leds_start();

// task: changes brightness continuosly
void taskBrightness(void *pvParameters);

// task: applies brightness to single LED
void taskControlLed(void *pvParameters);

// toggles TTC_LED2 between on and off
void toggleLED();

// fast output of 16 bit value onto parallel port
void parallelOutput(u16_t Value);

// manually polls current value of TTC_SWITCH1
void pollSwitch1();

// interrupt service routine: automatically called when switch is pressed
void isr_Switch1(physical_index_t PhysicalIndex, void* Argument);

//} Function prototypes

#endif //EXAMPLE_LEDS
