#ifndef EXAMPLE_THREADING
#define EXAMPLE_THREADING

/*{ TemplateName::.h ************************************************
 
 Empty template for new header files.
 Copy and adapt to your needs.
 
}*/
//{ Defines/ TypeDefs ****************************************************

//} Defines
//{ Includes *************************************************************

 // Basic set of helper functions
#include "ttc_basic.h"
#include "ttc_gpio.h"
#include "ttc_task.h"
#include "ttc_queue.h"
#include "ttc_gfx.h"

//} Includes
//{ Structures/ Enums ****************************************************


typedef struct { // ets_ConfigLED_t
  ttc_Port_t Port;       // port where LED is connected
  u16_t OnPeriod;        //  on time of LED1 (usecs)
  u16_t OffPeriod;       // off time of LED1 (usecs)
} ets_ConfigLED_t;

typedef struct { // et_PrintJob_t
    // Column == -1 | Row == -1: continue at last position
    s8_t Column;   // >= 0: Column to move cursor to
    s8_t Row;      // >= 0: Row to move cursor to
    char Text[10]; // zero terminated string to print
    s32_t Value;   // passed as value to printf
} et_PrintJob_t;

/** data being sent to taskProducer()/ taskConsumer() */
typedef struct { // ets_ThreadInfo_t
        ttc_MutexHandle Lock_Init;                   // used to lock this structure during task initialisation
    ttc_SemaphoreHandle SemaphoreProducer2Consumer;  // semaphore to consumer tasks
        ttc_QueueHandle QueuePrint;                  // queue to print on display
                 int DisplayRow;                     // row in display used for debug output
     ets_ConfigLED_t ConfigLED;                      // led port + brightness
} ets_ThreadInfo_t;

//} Structures/ Enums
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************

// basic initialization (called from threading_start() )
void threading_init();

// run demo
void threading_start();

/** graphical display of current queue size
 * @param Semaphore   Semaphore/ Queue which size should be displayed
 * @param QueuePrint  queue to print to
 */
void displayQueue(ttc_SemaphoreHandle Semaphore, ttc_QueueHandle QueuePrint);

// produces tasks to do
void taskProducer(void *pvParameters);

// consumes + executes tasks to do
void taskConsumer(void *pvParameters);

// adds given data as new print job to given queue
// Will block if queue is full
void queuePrintJob(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text);
void queuePrintJobV(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s32_t Value);

// central print task (display cannot be used from multiple tasks
void taskPrint(void *pvParameters);


//} Function prototypes

#endif //EXAMPLE_THREADING
