#!/bin/bash

#{  Install script for Open On-Chip Debugger tool made by Dominic Rath.
#  Script written by Gregor Rebel 2010-2011.
#  OpenOCD - On Chip Debugger
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64
# openSuSE 12.1 x86
#}

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="999_open_ocd+swd"

# binary to create
OpenOCD_Binary="openocd_swd"

dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

InstallInHomeDir="1"
if [ "$InstallInHomeDir" == "1" ]; then
  Prefix=`pwd`  # install into subfolder of ToolChain/ for current user only
else
  findPrefix    # install for all users
fi

InstallDir=`pwd`

if [ ! -e OK.Documentation ]; then #{
  DocFolder="../../Documentation/Interfaces"
  Error=""
  dir $DocFolder
  OldDir=`pwd`
  cd ${DocFolder}
  #X (website is very slow) - wget -rnp -nd -nc http://openocd.berlios.de/doc/html/index.html
  
  cd "$OldDir"

  if [ "$Error" == "" ]; then
    echo "Documentation complete: $INSTALLPATH."
    touch OK.Documentation
  fi
fi #}
if [ ! -e OK.Install ]; then #{

  echo "installing in $INSTALLPATH ..."
  
  if [ ! -e OK.Packages ]; then #{
    installPackage texlive
    installPackage texinfo
    touch OK.Packages
  fi #}
  
  DirName="libswd"
  
  gitClone 	git://repo.or.cz/openocd/libswd.git $DirName
  if [ ! -d $DirName ]; then
    echo "$0 - ERROR: Cannot download libwsd repositiory!"
    exit 10
  fi
  
  cd "${DirName}/"
  echo "compiling $DirName ----------------------------------------------"
  echo "clean..." && make clean >clean.log 2>&1

  # workaround: avoid make error "openocd.texi:12: @include `version.texi': Datei oder Verzeichnis nicht gefunden." 
  touch doc/version.texi
  
  echo "bootstrap >bootstrap.log..." &&  ./bootstrap >bootstrap.log 2>&1

  #{ workaround: avoid error message during configure because of unknown command AM_PROG_AR
  #replaceInFile configure.ac "AM_PROG_AR" "#AM_PROG_AR"
  
  replaceInFile submodules/libswd/configure.ac "AM_PROG_AR" "#AM_PROG_AR"
  cd submodules/libswd
  echo "`pwd`/autogen.sh .." && ./autogen.sh >autogen.log 2>&1
  cd ../..
 
  OK=""
  echo "bootstrap >bootstrap.log..." &&  ./bootstrap >bootstrap.log 2>&1  && OK="1"
  IncludeDir="`pwd`/../../999_libftdi/"
  if [ "$OK" == "" ]; then
    echo "Error during bootstrapping ${DirName} inside `pwd`!"
    exit 10
  fi
  
  #}workaround
  
  OK=""
  
  # --enable-maintainer-mode --enable-ft2232_ftd2xx 
  # --enable-stlink (implementation broken)
  Options="--prefix=$Prefix --enable-ioutil --enable-oocd_trace --enable-jlink --enable-ft2232_libftdi"
  
  echo "`pwd` > ./configure $Options" >configure.log
  echo ""                            >>configure.log
  echo "`pwd` > ./configure $Options"
  ./configure $Options >>configure.log 2>&1  && OK="1"

  if [ "$OK" == "" ]; then
    echo "Error during configuring ${DirName}!"
    exit 10
  fi
  for Makefile in $( find ./ -name Makefile ); do
    echo "replacing Werror in $Makefile .."
    replaceInFile $Makefile "-Werror" " "
  done
  if [ "$AmountCPUs" == "" ]; then
    AmountCPUs="1"
  fi
  AmountJobs=$(( $AmountCPUs * 2 ))
  rm -f OK.Compile
  echo "make -j${AmountJobs}  >make.log..."      && make -j$AmountJobs    >make.log 2>&1 && \
  echo "install   >install.log..."               && sudo make install  >install.log 2>&1 && \
  touch OK.Compile
  echo "pdf       >pdf.log..."      && make pdf -j$AmountJobs >pdf.log 2>&1
  dir ../${DocFolder}/OpenOCD+SWD
  cp -v doc/*.pdf ../${DocFolder}/OpenOCD+SWD/
  
  if [ ! -e OK.Compile ]; then #{
    echo "Error during compiling ${DirName}!"
    echo "Check log files:"
    find `pwd`/ -name "*.log" -exec echo "  less {}" \;
    find `pwd`/ -name "*.err" -exec echo "  less {}" \;
    exit 10
  fi #}

  cd ..
  sudo chown -R $USER: share/openocd/scripts
  
  if [ -e "bin/openocd" ]; then
      IsInstalled=`$Prefix/bin/openocd --version  2>&1 | grep Debugger`
      if [ "$IsInstalled" != "" ]; then
        sudo chmod 4755 $Prefix/bin/openocd
        echo "Installed version: $IsInstalled"
        addLine ../../cleanup.sh "rm -Rf `pwd`/${DirName}/"
        cd "$InstallDir"
        
        echo "" >OK.Install
      else
        echo "$0 - ERROR: Cannot find binary '$Prefix/bin/openocd --version' !"
      fi
  else
    echo "$0 - ERROR: Cannot find binary `pwd`/bin/openocd !"
  fi
fi #} [ ! -e OK.Install ]; then
if [ -e OK.Install ]; then #{

  ConfigFolder="$InstallDir/share/openocd/scripts/interface/"
  OldDir=`pwd`
  cd "$ConfigFolder"
    
  sudo chown $USER: .
  # ensure that current folder is writable
  cat <<END_OF_CFG >wsn3_oocdlink-h.cfg #{
#
# Joern Kaipf's OOCDLink
# Modified by Gregor Rebel for Wireless Sensor Node v3 Programming Adapter
#
# http://www.joernonline.de/contrexx2/cms/index.php?page=126
#

interface ft2232
ft2232_device_desc "WSN3 OOCDLink-H A"
ft2232_layout oocdlink
ft2232_vid_pid 0x0403 0xbafa
adapter_khz 200
reset_config trst_and_srst separate trst_push_pull srst_push_pull

END_OF_CFG
#}

  Interfaces=`ls *.cfg`
  cd "$OldDir"
  if [ ! -e ../../Template/configs/interface.cfg ]; then #{
    cat <<END_OF_CFG >../../Template/configs/interface.cfg #{
#!/bin/bash
#
# (c) Gregor Rebel 2010-2011
#

# Choose config file that suits to your program adapter
#
# Configfiles are read from folder
# $ConfigFolder

# remove leading # from desired interface to use: 
Interface="olimex-arm-usb-ocd-h.cfg"
END_OF_CFG
#}

    for Interface in $Interfaces; do
      echo "# Interface=\"$Interface\"" >>../../Template/configs/interface.cfg
    done
  fi #}
  if [ "$InstallInHomeDir" == "1" ]; then
    createLink ../$INSTALLPATH/bin/openocd ../bin/$OpenOCD_Binary
    addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH;  ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/share/openocd/scripts/  $INSTALLPATH"

    #X addLine ../scripts/SourceMe.sh "export PATH=\"\$HOME/Source/ToolChain_STM32/ToolChain/$INSTALLPATH/bin/:\$PATH\" #$0"
  else
    createLink $Prefix/bin/openocd ../bin/$OpenOCD_Binary
    #X addLine ../scripts/SourceMe.sh "export PATH=\"$Prefix/bin/:\$PATH\" #$0"
  fi
  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

cd ..

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
