#!/bin/bash
#
#  Install script for open source GCC G++ compiler for CortexM3 MicroControllers. 
#  Script written by Gregor Rebel 2010-2011.
#
# Reference: http://michaldemin.wordpress.com/2010/02/09/arm-toolchain-newlib-part1/
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

export PATH="`pwd`/scripts:$PATH"
source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="050_compiler_gcc_cortex_m3"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Install ]; then #{

echo "installing in $INSTALLPATH ..."

CleanUpBatch="cleanup.sh"

export TOOLPATH="`pwd`/bin/"
export ARMTARGET="arm-softfloat-elf"


InstallFile="binutils.tar.bz2"
if [ ! -e "OK.$InstallFile" ]; then #{
  rm -Rf "binutils-*"
  getNewestFile http://ftp.gnu.org/gnu/binutils/ "binutils-..*bz2\$" "$InstallFile"
  if [ -e "$InstallFile" ]; then #{
    tar xjf "$InstallFile"
    Dir=`find ./ -maxdepth 1 -type d -name "binutils-*"`
    if [ "$Dir" != "" ]; then
      echo "compiling $NewestFile ..."
      ln -s $Dir gmp
      cd $Dir
      mkdir build
      cd build
      ../configure --target=$ARMTARGET --prefix=$TOOLPATH --enable-interwork --enable-multilib --with-gnu-as --with-gnu-ld --disable-nls >../log.configure 2>&1 && \
      make -j$AmountCPUs all >../log.compile 2>&1 && \
      make install >../log.install 2>&1
      cd ..
      cd ..
    fi
    if [ -e bin/bin/arm-softfloat-elf-objdump ]; then
      rm -Rf $Dir
      touch "OK.$InstallFile"
      echo "Successfully compiled $NewestFile"
    fi
  else
    echo "$0 - ERROR: missing file '$InstallFile'!"
    exit 10
  fi #}
fi #}

InstallFile="gmp.tar.bz2"
if [ ! -e "OK.$InstallFile" ]; then #{
  findNewest "ftp://ftp.gmplib.org/pub/" "gmp-.+" "$InstallFile"
  getFile "ftp://ftp.gmplib.org/pub/${NewestDir}/" "${NewestDir}.tar.bz2" "$InstallFile"

  if [ -e "$InstallFile" ]; then #{
    tar xjf "$InstallFile"
    Dir=`find ./ -maxdepth 1 -type d -name "gmp-*"`
    if [ "$Dir" != "" ]; then
      echo "compiling $NewestDir ..."
      cd $Dir
      ./configure --enable-cxx >log.configure 2>&1 && \
      make -j$AmountCPUs >log.compile 2>&1 && \
      make -j$AmountCPUs check >log.install 2>&1 && \
      touch ../OK.$InstallFile
      cd ..
    fi
    if [ -e OK.$InstallFile ]; then
      echo "compiled successfully: $NewestDir"
    else
      echo "$0 - ERROR: failed to install '$InstallFile'!"
      exit 11
    fi
  else
    echo "$0 - ERROR: missing file '$InstallFile'!"
    exit 10
  fi #}
fi #}
Dir_GMP="`pwd`/gmp/"

InstallFile="mpfr.tar.bz2"
if [ ! -e "OK.$InstallFile" ]; then #{
  getNewestFile "http://www.mpfr.org/mpfr-current/" "mpfr-.+\.tar\.bz2\$" "$InstallFile"

  if [ -e "$InstallFile" ]; then #{
    tar xjf "$InstallFile"

    Dir=`find ./ -maxdepth 1 -type d -name "mpfr-*"`
    if [ "$Dir" != "" ]; then
      echo "compiling $Dir ..."
      ln -s $Dir mpfr
      cd $Dir
      get http://www.mpfr.org/mpfr-current/ allpatches
      if [ -e allpatches ]; then
        patch -N -Z -p1 < allpatches
      fi
      Configure="./configure --prefix=$TOOLPATH --with-gmp-build=$Dir_GMP"
      Compile="make -j$AmountCPUs"
      
      make clean
      echo $Configure && $Configure >log.configure 2>&1 && \
      echo $Compile && $Compile >log.compile 2>&1 && \
      make -j$AmountCPUs check >log.compile 2>&1 && \
      make install >log.install 2>&1 &&
      touch "../OK.$InstallFile"
      
      cd ..
    fi
    
    if [ -e "OK.$InstallFile" ]; then
      echo "successfully compiled $Dir"
    else
      echo "$0 - ERROR: failed to install '$InstallFile' (Check log-files inside `pwd`/$Dir/)!"
      exit 11
    fi
  else
    echo "$0 - ERROR: missing file '$InstallFile'!"
    exit 10
  fi #}
fi #}
Dir_MPFR="`pwd`/mpfr/"


InstallFile="mpc.tar.gz"
if [ ! -e "OK.$InstallFile" ]; then #{
  getNewestFile "http://www.multiprecision.org/index.php?prog=mpc&page=download" "Version" "$InstallFile"

  if [ -e "$InstallFile" ]; then #{
    tar xzf "$InstallFile"
    Dir=`find ./ -maxdepth 1 -type d -name "mpc-*"`
    if [ "$Dir" != "" ]; then
      echo "compiling $Dir ..."
      ln -s $Dir mpc
      cd $Dir

      export CFLAGS='-Wall -Wmissing-prototypes -Wpointer-arith -Wall -g  -Os'
      Configure="./configure --prefix=$TOOLPATH --with-gmp=$Dir_GMP"
      Compile="make -j$AmountCPUs"
      
      echo $Configure && $Configure >log.configure 2>&1
      
      # compilation of mpc-1.0.1 with gcc-4.6.3 failed with option -O3, but succeeded with option -Os
      replaceInFile "libtool" "-O3" "-Os"
      replaceInFile "Makefile" "-O3" "-Os"

      echo $Compile && $Compile >log.compile 2>&1 && \
      make -j$AmountCPUs check >log.compile 2>&1 && \
      make install >log.install 2>&1 &&
      touch "../OK.$InstallFile"

      cd ..
    fi
    if [ -e OK.$InstallFile ]; then
      echo "successfully compiled $Dir"
    else
      echo "$0 - ERROR: failed to install '$InstallFile'!"
      exit 11
    fi
  else
    echo "$0 - ERROR: missing file '$InstallFile'!"
    exit 10
  fi #}
fi #}
Dir_MPC="`pwd`/mpc/"

InstallFile="ppl.tar.gz"
if [ ! -e "OK.$InstallFile" ]; then #{
  getNewestFile "http://bugseng.com/products/ppl/download/ftp/releases/LATEST/" "ppl-.+\.tar\.gz\$" "$InstallFile"

  if [ -e "$InstallFile" ]; then #{
    tar xzf "$InstallFile"
    Dir=`find ./ -maxdepth 1 -type d -name "ppl-*"`
    if [ "$Dir" != "" ]; then
      echo "compiling $NewestFile ..."
      ln -s $Dir ppl
      cd $Dir
      mkdir build
      cd build
      Configure="../configure --prefix=$TOOLPATH"
      Compile="make -j$AmountCPUs"
      echo $Configure && $Configure  >../log.configure 2>&1 && \
      echo $Compile   && $Compile    >../log.compile   2>&1 && \
      make install                   >../log.install   2>&1 && \
      touch "../..//OK.$InstallFile"
      
      cd ..
      cd ..
    fi
    if [ -e OK.$InstallFile ]; then
      echo "successfully compiled $Dir"
    else
      echo "$0 - ERROR occured during compiling $InstallFile (check log-files in `pwd`/$Dir/)!"
      exit 10
    fi
  else
    echo "$0 - ERROR: missing file '$InstallFile'!"
    exit 10
  fi #}
fi #}

InstallFile="cloog-ppl"
if [ ! -e "OK.$InstallFile" ]; then #{
  gitClone git://repo.or.cz/cloog-ppl.git cloog-ppl.git

  Dir="cloog-ppl"
  if [ -d "$Dir" ]; then
    echo "compiling $Dir ..."
    cd $Dir
    
    mkdir m4
    Configure="./configure --prefix=$TOOLPATH"
    Compile="make -j$AmountCPUs"
    echo $Configure && $Configure  >../log.configure 2>&1 && \
    echo $Compile   && $Compile    >../log.compile   2>&1 && \
    make install                   >../log.install   2>&1 && \
    touch "../..//OK.$InstallFile"

    cd ..
  fi
  
  if [ -e OK.$InstallFile ]; then
    echo "successfully compiled $Dir"
  else
    echo "$0 - ERROR occured during compiling $InstallFile (check log-files in `pwd`/$Dir/)!"
    exit 10
  fi
fi #}


exit 0 #D

InstallFile="gcc.tar.gz"
if [ ! -e "OK.$InstallFile" ]; then #{
  rm -Rf "gcc-*"
  findNewest "ftp://ftp.gwdg.de/pub/misc/gcc/releases/" "gcc-.*"
  getFile "ftp://ftp.gwdg.de/pub/misc/gcc/releases/${NewestDir}/" "${NewestDir}.tar.bz2" "$InstallFile"
  if [ -e "$InstallFile" ]; then #{
    tar xjf "$InstallFile"
    Dir=`find ./ -maxdepth 1 -type d -name "gcc-*"`
    if [ "$Dir" != "" ]; then
      echo "compiling $NewestFile ..."
      ln -s $Dir gcc
      cd $Dir
      replaceInFile "gcc/config/arm/elf.h" \
                    "#define SUBTARGET_ASM_FLOAT_SPEC" \
                    "#define SUBTARGET_ASM_FLOAT_SPEC %{mhard-float:-mfpu=fpa} %{!mhard-float:-mfpu=softfpa} "
           
      File="gcc/config/arm/t-arm-elf"
      
      replaceInFile "$File" \
                    "#MULTILIB_OPTIONS      += march=armv7" \
                    "MULTILIB_OPTIONS      += march=armv7"
      replaceInFile "$File" \
                    "#MULTILIB_DIRNAMES     += thumb2" \
                    "MULTILIB_DIRNAMES     += thumb2"
      replaceInFile "$File" \
                    "#MULTILIB_EXCEPTIONS   += march=armv7\* marm/\*march=armv7\*" \
                    "MULTILIB_EXCEPTIONS   += march=armv7\* marm/\*march=armv7\*"
      replaceInFile "$File" \
                    "#MULTILIB_MATCHES      += march?armv7=march?armv7-a" \
                    "MULTILIB_MATCHES      += march?armv7=march?armv7-a"
      replaceInFile "$File" \
                    "#MULTILIB_MATCHES      += march?armv7=march?armv7-r" \
                    "MULTILIB_MATCHES      += march?armv7=march?armv7-r"
      replaceInFile "$File" \
                    "#MULTILIB_MATCHES      += march?armv7=march?armv7-m" \
                    "MULTILIB_MATCHES      += march?armv7=march?armv7-m"
      replaceInFile "$File" \
                    "#MULTILIB_MATCHES      += march?armv7=mcpu?cortex-a8" \
                    "MULTILIB_MATCHES      += march?armv7=mcpu?cortex-a8"
      replaceInFile "$File" \
                    "#MULTILIB_MATCHES      += march?armv7=mcpu?cortex-r4" \
                    "MULTILIB_MATCHES      += march?armv7=mcpu?cortex-r4"
      replaceInFile "$File" \
                    "#MULTILIB_MATCHES      += march?armv7=mcpu?cortex-m3" \
                    "MULTILIB_MATCHES      += march?armv7=mcpu?cortex-m3"

      mkdir build
      cd build
      createLink ../../bin  mpfr
      createLink ../../bin  mpc
      createLink ../../bin  gmp
      createLink ../../bin  ppl

      Configure="../configure --target=$ARMTARGET --prefix=$TOOLPATH --enable-interwork --enable-multilib --enable-languages='c,c++' --with-newlib --without-headers --disable-shared --with-gnu-as --with-gnu-ld --with-float=soft  --with-gmp=gmp --with-mpfr=mpfr --with-mpc=mpc --with-ppl=ppl"
      Compile="make -j$AmountCPUs all-gcc"
      #../configure --enable-obsolete --target=$ARMTARGET --prefix=$TOOLPATH --enable-interwork --enable-multilib --enable-languages="c,c++" --with-newlib --without-headers --disable-shared --with-gnu-as --with-gnu-ld --with-float=soft  --with-gmp=gmp --with-mpfr=mpfr --with-mpc=mpc >../log.configure 2>&1 && \
      echo $Configure && $Configure  >../log.configure 2>&1 && \
      echo $Compile   && $Compile    >../log.compile   2>&1 && \
      make -j$AmountCPUs            >>../log.compile   2>&1 ; \
      make install >../log.install 2>&1 && \
      touch "../OK.$InstallFile"
      cd ..
      
      cd ..
    fi
    if [ -e OK.$InstallFile ]; then
      echo "successfully compiled $Dir"
    else
      echo "$0 - ERROR occured during compiling $InstallFile (check log-files in `pwd`/$Dir/)!"
      exit 10
    fi
  else
    echo "$0 - ERROR: missing file '$InstallFile'!"
    exit 10
  fi #}
fi #}

exit 0

InstallFile="newlib.tar.gz"
if [ ! -e "OK.$InstallFile" ]; then #{
  getNewestFile "ftp://sources.redhat.com/pub/newlib/" "newlib-.+\.tar\.gz\$" "$InstallFile"

  if [ -e "$InstallFile" ]; then #{
    tar xjf "$InstallFile"
    Dir=`find ./ -maxdepth 1 -type d -name "gcc-*"`
    if [ "$Dir" != "" ]; then
      echo "compiling $NewestFile ..."
      ln -s $Dir gcc
      cd $Dir

      cd ..
    fi
    
    if [ -e OK.$InstallFile ]; then
      echo "successfully compiled $Dir"
    else
      echo "$0 - ERROR occured during compiling $InstallFile (check log-files in `pwd`/$Dir/)!"
      exit 10
    fi
  else
    echo "$0 - ERROR: missing file '$InstallFile'!"
    exit 10
  fi #}
fi #}

exit 0
#X getFile "http://ftp.gnu.org/gnu/binutils/"                   "binutils-2.20.1.tar.bz2"
#X getFile "ftp://ftp.gwdg.de/pub/misc/gcc/releases/gcc-4.4.3/" "gcc-4.4.3.tar.bz2"
#X getFile "ftp://sources.redhat.com/pub/newlib/"               "newlib-1.18.0.tar.gz"


if [ -e "$BinFolder/arm-none-eabi-gcc" ]; then
    cat <<END_OF_MAKEFILE >../extensions/makefile.$INSTALLPATH
#
#                               ToolChain_STM32
#
#                         Extension to central makefile
#
#                   makefile written by Gregor Rebel 2010-2011
#

EXTENSION_$INSTALLPATH=1

END_OF_MAKEFILE

  echo "" >OK.Install
fi
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
  addLine ../scripts/SourceMe.sh    "export PATH=\"\$PATH:$BinFolder/\""
  addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH;           ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/arm-2010q1/                                   $INSTALLPATH"
  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
