#!/bin/bash

#
#  Install script for combined Microcontroller + 2,4GHz radio STM32 Wxxx 
#  with ARM Cortex M3 core.
#
#  Script written by Gregor Rebel 2012
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

CPURank="200"  # rank of cpu
LibsRank1="251" # rank of cpu libraries (simplemac)
LibsRank2="253" # rank of cpu libraries (hal)
LibsRank3="255" # rank of cpu libraries (std_peripheral)

INSTALL_PREFIX="cpu_stm32w1xx"
INSTALLPATH="${CPURank}_$INSTALL_PREFIX"
dir "$INSTALLPATH"
cd "$INSTALLPATH"


#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

  if [ ! -e OK.Documentation ]; then #{ download documentation
    Dir=`pwd`
    Get_Fails=""
    dir ../../Documentation/uC
    dir ../../Documentation/uC/STM32W1xx
    
    Error=""
    #{ download docs..
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/PROGRAMMING_MANUAL/ CD00280769.pdf PM0073-STM32W1xx_Programming_Manual.pdf                                         ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00262415.pdf UM0894-STM32W-SK_and_STM32W-EXT_starter_and_extension_kits.pdf                  ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00280375.pdf UM0978-Using_the_Simple_MAC_nodetest_application.pdf                            ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00264812.pdf UM0909-STM32W108xx_ZigBee_RF4CE_library.pdf                                     ../../Documentation/uC/STM32W1xx/
    #Not available anymore: getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00271698.pdf AN3211-Using_the_simulated_EEPROM_with_the_STM32W108_platform.pdf               ../../Documentation/uC/STM32W1xx/
    getFile http://www.po-star.com/public/uploads/                                              20120227121858_397.pdf AN3211-Using_the_simulated_EEPROM_with_the_STM32W108_platform.pdf               ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00282958.pdf AN3262-Using_the_over-the-air_bootloader_with_STM32W108_devices.pdf             ../../Documentation/uC/STM32W1xx/
    #Not available anymore: getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00269334.pdf AN3188-Preparing_custom_devices_for_the_STM32W108_platform.pdf                  ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00270908.pdf AN3206-PCB_design_guidelines_for_the_STM32W108_platform.                        ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   DM00024648.pdf AN3359-Low_cost_PCB_antenna_for_2.4GHz_radio_Meander_design.pdf                 ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00004479.pdf AN1709-EMC_design_guide_for_ST_microcontrollers.pdf                             ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00274443.pdf AN3218-Adjacent_channel_rejection_measurements_for_the_STM32W108_platform.pdf   ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATA_BRIEF/         CD00237927.pdf DB0837-High-performance_802.15.4_wireless_system-on-chip_databrief.pdf          ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/          CD00248316.pdf DS6473-High-performance_IEEE_802.15.4_wireless_system-on-chip_Datasheet.pdf     ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/SALES_AND_MARKETING_RESOURCES/MARKETING_PRESENTATIONS/PRODUCT_PRESENTATION/ stm32w_marketing_pres.pdf  STM32W-Product_presentation.pdf                      ../../Documentation/uC/STM32W1xx/
#}

    if [ "$Get_Fails" == "" ]; then
      echo "Documentation complete: $INSTALLPATH."
      touch OK.Documentation
    else
      echo "ERROR: Documentations not being received: $Get_Fails"
    fi
  fi #}
  if [ ! -e OK.Install ]; then #{       download + install library via zipped library
    # find -L $HOME/.wine/drive_c -name "*SimpleMAC-1*" -and -type d 2>/dev/null
    
    Link="SimpleMac-Firmware"
    if [ ! -d "$Link" ]; then #{ download + extract archive + run microsoft installer to obtain SimpleMAC library
      echo "missing folder '$Link'"
      File="STM32W108_SimpleMAC_firmware"
      getFile http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/ stm32w_simplemac_fw.zip $File.zip
      unZIP "e" "${File}.zip"
      
      LinkTarget=`find ./ -maxdepth 1 -iname "STM32W108xx_SimpleMAC_*" -type d`
      createLink $LinkTarget $Link
    fi #}
    if [ -d "$Link" ]; then   #{ create links to libraries
      MissingFiles="" # updated by createLink()
      createLink "$Link/Libraries/SimpleMAC" SimpleMac-Library
      createLink "$Link/Utilities/STM32W108xx_HAL_Driver" hal_library
      createLink "$Link/Libraries/STM32W108xx_StdPeriph_Driver" stm_std_peripherals
      if [ "$MissingFiles" == "" ]; then
        touch OK.Install
      else
        echo "$0 - ERROR: missing files: $MissingFiles"
      fi
    else  
      echo "$0 - ERROR: Cannot find installed SimpleMAC-library in $Link !"
      exit 0
    fi #}
  fi #}
  if [ ! -e OK.Install ]; then #{       download + install library via Windows Installer package (DEPRECATED)
    # find -L $HOME/.wine/drive_c -name "*SimpleMAC-1*" -and -type d 2>/dev/null
    
    if [ ! -e OK.Wine ]; then #{      install required software packages

  # make sure that eula is accepted automatically    
cat <<END_OF_TEXT |  sudo /usr/bin/debconf-set-selections
ttf-mscorefonts-installer msttcorefonts/accepted-mscorefonts-eula select true
END_OF_TEXT

    installPackage wine
    Wine=`which wine 2>/dev/null`
    if [ "$Wine" != "" ]; then 
      touch OK.Wine
    else
      echo "$0 - ERROR: Cannot install wine!"
      exit 10
    fi
  fi #}

    Folder="$HOME/.wine/drive_c/Program Files (x86)/STMicroelectronics"
    if [ ! -d "$Folder" ]; then
      Folder="$HOME/.wine/drive_c/Program Files/STMicroelectronics"
    fi
    if [ ! -d "$Folder" ]; then #{ download + extract archive + run microsoft installer to obtain SimpleMAC library
      echo "missing folder '$Folder'"
      File="STM32W108_SimpleMAC_firmware"

      getFile http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/ st_zigbee_simplemac.zip $File.zip
      unZIP "e" "${File}.zip"
      echo "Starting windows installer for STM32Wxxx SimpleMAC-library. Please install into default directory (press OK in every dialog)!"
      Archive=`ls *SimpleMAC*exe`
      ln -s "$Archive" SimpleMAC.exe
      if [ -e SimpleMAC.exe ]; then
        wine ./SimpleMAC.exe
      else
        echo "$0 - ERROR: Cannot find SimpleMAC.exe!"
        exit 13
      fi
    fi #}
    if [ -d "$Folder" ]; then   #{ create links to libraries
     
      Version=`ls "$Folder/"`
      MissingFiles="" # updated by createLink()
      createLink "$Folder/$Version" .
      createLink "$Version" SimpleMac-Library
      createLink "$Version/STM32W108/hal/" hal_library

      if [ "$MissingFiles" == "" ]; then
        touch OK.Install
      else
        echo "$0 - ERROR: missing files: $MissingFiles"
      fi
    #}
    else  
      echo "$0 - ERROR: Cannot find installed SimpleMAC-library in $Folder !"
      exit 0
    fi
  fi #}
  if [ ! -e OK.PatchFiles ]; then #{    patch source files
    ERROR=""
    PatchFiles=`find SimpleMac-Library/ -follow -name "*.c" -exec grep -l "#include BOARD_HEADER" {} \;`
    for PatchFile in $PatchFiles; do #{
      if [ ! -e $PatchFile ]; then
        ERROR="1"
        echo "ERROR: File not found: '$PatchFile'!"
      else
        echo "replacing BOARD_HEADER in $PatchFile.."
        replaceInFile $PatchFile BOARD_HEADER  '"hal/micro/cortexm3/stm32w108/board.h" //BOARD_HEADER (patched for TheToolChain)'
      fi
    done #}

    PatchFiles=`find SimpleMac-Library/ -follow -name "*.c" -exec grep -l "#include PLATFORM_HEADER" {} \;`
    for PatchFile in $PatchFiles; do #{
      if [ ! -e $PatchFile ]; then
        ERROR="1"
        echo "ERROR: File not found: '$PatchFile'!"
      else
        echo "replacing PLATFORM_HEADER in $PatchFile.."
        replaceInFile $PatchFile PLATFORM_HEADER  '"micro/cortexm3/compiler/gnu.h" //PLATFORM_HEADER (patched for TheToolChain)'
      fi
    done #}
    
    PatchFiles=`find SimpleMac-Library/ -follow -name "*" -exec grep -l "RAM_region" {} \;`
    for PatchFile in $PatchFiles; do #{
      if [ ! -e $PatchFile ]; then
        ERROR="1"
        echo "ERROR: File not found: '$PatchFile'!"
      else
        echo "replacing RAM_region in $PatchFile.."
        replaceInFile $PatchFile RAM_region  'ram'
      fi
    done #}

    PatchFiles=`find SimpleMac-Library/ -follow -name "*" -exec grep -l "ROM_region" {} \;`
    for PatchFile in $PatchFiles; do #{
      if [ ! -e $PatchFile ]; then
        ERROR="1"
        echo "ERROR: File not found: '$PatchFile'!"
      else
        echo "replacing RAM_region in $PatchFile.."
        replaceInFile $PatchFile ROM_region  'rom'
      fi
    done #}
    
    PatchFiles=`find SimpleMac-Library/ -follow -name "*" -exec grep -l "FIB_region" {} \;`
    for PatchFile in $PatchFiles; do #{
      if [ ! -e $PatchFile ]; then
        ERROR="1"
        echo "ERROR: File not found: '$PatchFile'!"
      else
        echo "replacing FIB_region in $PatchFile.."
        replaceInFile $PatchFile FIB_region  'fib'
      fi
    done #}

    
    if [ "$ERROR" == "" ]; then
      touch OK.PatchFiles
    fi
  fi #}
  if [   -e OK.Install ]; then #{       create activate-scripts + makefiles
    ConfigFile="../999_open_ocd/share/openocd/scripts/target/stm32w1xx.cfg"
    if [ ! -e "$ConfigFile" ]; then
      echo "creating cfg-file '`pwd`/$ConfigFile'.."
      cat <<'END_OF_CONFIG' >$ConfigFile #{
# script for stm32w108

if { [info exists CHIPNAME] } {
   set  _CHIPNAME $CHIPNAME
} else {
   set  _CHIPNAME em357
}

if { [info exists ENDIAN] } {
   set  _ENDIAN $ENDIAN
} else {
   set  _ENDIAN little
}

# Work-area is a space in RAM used for flash programming
# By default use 16kB
if { [info exists WORKAREASIZE] } {
   set  _WORKAREASIZE $WORKAREASIZE
} else {
   #set  _WORKAREASIZE 0x4000
   set  _WORKAREASIZE 0x2000
}

# JTAG speed should be <= F_CPU/6. F_CPU after reset is 8MHz, so use F_JTAG = 1MHz
adapter_khz 200

adapter_nsrst_delay 400
jtag_ntrst_delay 100

#verify_ircapture disable 
#verify_jtag disable


#jtag scan chain
if { [info exists CPUTAPID ] } {
   set _CPUTAPID $CPUTAPID
} else {
  # See STM Document RM0008
  # Section 26.6.3
   set _CPUTAPID 0x3ba00477
}
jtag newtap $_CHIPNAME cpu -irlen 4 -ircapture 0x1 -irmask 0xf -expected-id $_CPUTAPID

if { [info exists BSTAPID ] } {
   # FIXME this never gets used to override defaults...
   set _BSTAPID $BSTAPID
} else {
  # See STM Document RM0008
  # Section 29.6.2
  # Low density devices, Rev A
  #set _BSTAPID1 0x06412041
  # Medium density devices, Rev A
  #set _BSTAPID2 0x06410041
  # Medium density devices, Rev B and Rev Z
  #set _BSTAPID3 0x16410041
  #set _BSTAPID4 0x06420041
  # High density devices, Rev A
  #set _BSTAPID5 0x06414041
  #set _BSTAPID5 0x06418041
  # Connectivity line devices, Rev A and Rev Z
  #set _BSTAPID6 0x06418041
  # XL line devices, Rev A
  #set _BSTAPID7 0x06430041
  
  
  set _BSTAPID1 0x269a862b
 set _BSTAPID 0x269a862b

}

jtag newtap $_CHIPNAME bs -irlen 4 -ircapture 0x0e -irmask 0xf  -expected-id $_BSTAPID

	#	jtag newtap $_CHIPNAME bs -irlen 5 -expected-id $_BSTAPID1 \
#	-expected-id $_BSTAPID2 -expected-id $_BSTAPID3 \
#	-expected-id $_BSTAPID4 -expected-id $_BSTAPID5 \
#	-expected-id $_BSTAPID6 -expected-id $_BSTAPID7



set _TARGETNAME $_CHIPNAME.cpu
#$_TARGETNAME configure -work-area-phys 0x20000000 -work-area-size $_WORKAREASIZE -work-area-backup 0

target create $_TARGETNAME cortex_m3 -endian $_ENDIAN -chain-position $_TARGETNAME

$_TARGETNAME configure -work-area-phys 0x20000000 -work-area-size $_WORKAREASIZE -work-area-backup 0

# flash size will be probed
set _FLASHNAME $_CHIPNAME.flash
#flash bank $_FLASHNAME stm32f1x 0x08000000 0 0 0 $_TARGETNAME
flash bank $_FLASHNAME em357 0x08000000 0 0 0 $_TARGETNAME

# if srst is not fitted use SYSRESETREQ to
# perform a soft reset
cortex_m3 reset_config sysresetreq

END_OF_CONFIG
#}
    else
      echo "cfg-file already exists: '`pwd`/$ConfigFile'.."
    fi

    ConfigFile="../999_open_ocd/share/openocd/scripts/target/stm32w1xx_flash.cfg"
    if [ ! -e "$ConfigFile" ]; then
      echo "creating cfg-file '`pwd`/$ConfigFile'.."
      cat <<'END_OF_CONFIG' >$ConfigFile #{
# flash script for stm32w108


# finish configuration stage + proceed to run stage
init

# hardware reset processor with halt (2 times is safer than only once)
soft_reset_halt
soft_reset_halt

# check target state em357 
poll

# list all found flash banks
flash banks

# identify the flash
flash probe 0

em357 unlock 0

# erasing all flash
em357 mass_erase 0

# write file to flash number 0
flash write_bank 0 main.bin 0

# hardware processor reset with run
reset run

#shutdown openocd daemon
shutdown

END_OF_CONFIG
#}
    else
      echo "cfg-file already exists: '`pwd`/$ConfigFile'.."
    fi

    Name="${CPURank}_${INSTALL_PREFIX}" #{                   makefile + activate-script for CPU STM32W1xx
    createExtensionMakefileHead ${Name} #{
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File #{

ARM_CPU     =arm7tdmi
TARGET      =-mcpu=cortex-m3 -mthumb
TARGET_ARCHITECTURE_CM3=1
TARGET_ARCHITECTURE_EM357=1
TARGET_ARCHITECTURE_PREFIX=STM32
COMPILE_OPTS += -DTARGET_ARCHITECTURE_PREFIX=EM357
COMPILE_OPTS += -DTARGET_ARCHITECTURE_EM357
COMPILE_OPTS += -DTARGET_ARCHITECTURE_EM357
COMPILE_OPTS += -DTARGET_ARCHITECTURE_CM3
COMPILE_OPTS += -DTARGET_DATA_WIDTH=32


# list of available devices (activate only one!)
STM32W108   =1       # STM32W Device

# check selected uC + add corresponding settings
ifdef STM32W108
  ifdef uCONTROLLER
    ERROR: STM32W108=1 - Only one uController may be selected!
  endif
  uCONTROLLER=STM32W108
  COMPILE_OPTS += -DuCONTROLLER=STM32W108
  COMPILE_OPTS += -DSTM32W108
  MAIN_OBJS += em357_basic.o crt_stm32w108.o context-switch.o spmr.o micro-common-internal.o micro-common.o
  

endif

COMPILE_OPTS += -Icommon/em357/
vpath %.c common/em357/

# some settings which may be required by HAL or SimpleMAC
COMPILE_OPTS += -DCORTEXM3_STM32W108
COMPILE_OPTS += -DPHY_STM32W108XX -DDISABLE_WATCHDOG -DCORTEXM3_STM32W108xB -DCORTEXM3

# define linker scripts to use
#X LDFLAGS+= -Tadditionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/gnu-stm32w108xB.ld

#X LDFLAGS+= -Tconfigs/memory.ld
LDFLAGS+= -Tconfigs/em357.ld
LDFLAGS+= -Tadditionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/gnu-stm32w108.ld

# define rules for assembler-files which may be compiled into objects
context-switch.o: context-switch.s
spmr.o: spmr.s



END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{
    createActivateScriptHead $File $0 "CPU STM32W1xx"
    cat <<END_OF_ACTIVATE >>$File #{

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

# create link to target script being required by openocd to initialize target cpu
cd \$Dir_Configs
createLink ../additionals/999_open_ocd/target/stm32w1xx.cfg       openocd_target.cfg
createLink ../additionals/999_open_ocd/target/stm32w1xx_flash.cfg openocd_flash.cfg
cd ..

cd "\$Dir_Additionals"
rm -fv ${CPURank}_cpu_* ${LibsRank1}_cpu_* ${LibsRank2}_cpu_*
createLink \$HOME/Source/ToolChain_STM32/ToolChain/$INSTALLPATH/SimpleMac-Library/  $INSTALLPATH
cd ..

END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}
    #}
    Name="${LibsRank1}_${INSTALL_PREFIX}_simple_mac" #{      makefile + activate-script for SimpleMAC-library
     createExtensionMakefileHead ${Name} #{
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File #{

EM357_SIMPLEMAC = 1
INCLUDE_DIRS += -I additionals/${LibsRank1}_cpu_stm32w1xx_SimpleMAC_library/include/

# SimpleMAC library is only available as precompiled binary :-(
#X LD_LIBS += -mthumb additionals/${LibsRank1}_cpu_stm32w1xx_SimpleMAC_library/library/libsimplemac-library-gnu.a
LD_LIBS += additionals/${LibsRank1}_cpu_stm32w1xx_SimpleMAC_library/library/libsimplemac-library-gnu.a

COMPILE_OPTS += -DTARGET_ARCHITECTURE_EM357

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{
    createActivateScriptHead $File $0 "STM32W1xx Simple MAC Library - provides access to on-chip transceiver"
    cat <<END_OF_ACTIVATE >>$File #{

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

cd "\$Dir_Additionals"
createLink $INSTALLPATH/STM32W108/simplemac/                                   ${Name}_SimpleMAC_library
cd ..

activate.${CPURank}_cpu_stm32w1xx.sh QUIET \"\$0\"

END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}
    #}_simple_mac
    Name="${LibsRank2}_${INSTALL_PREFIX}_hal" #{             makefile + activate-script for HAL-library
    createExtensionMakefileHead ${Name} #{
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File #{

ifndef EM357_SIMPLEMAC
  COMPILE_OPTS += -DHAL_STANDALONE   # This is a simplified version of the full stCalibrateVref
                                     # It is meant to be used on build for hal only application not linked with
                                     # simplemac library. The full version is embedded in the simplemac library.
                                     # The simplified version does not support calibration when manufacturing tokens are missing
                                     # or radio boost mode is activated
                                     # File: micro-common-internal.c
endif

INCLUDE_DIRS += -I additionals/${CPURank}_cpu_stm32w1xx/STM32W108/

INCLUDE_DIRS += -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/ \
                -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/ \
                -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/ \
                -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/ \
                -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/compiler/ \
                -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/generic/compiler \
                -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/compiler

                INCLUDE_DIRS += -I additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/


#? vpath additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/

# where to find files of different types
INCLUDE_DIRS += -Iadditionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/
vpath %.c additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/
vpath %.c additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/
vpath %.c additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/
vpath %.s additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/
vpath %.s additionals/${LibsRank2}_cpu_stm32w1xx_hal_library/micro/cortexm3/

INCLUDE_DIRS += -Icommon/cm3
vpath %.c common/cm3

MAIN_OBJS += micro.o board.o clocks.o mfg-token.o adc.o flash.o nvm.o

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{
    createActivateScriptHead $File $0 "STM32W1xx Hardware Abstraction Layer library (HAL)"
    cat <<END_OF_ACTIVATE >>$File #{

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

cd "\$Dir_Additionals"
createLink $INSTALLPATH/hal_library/ ${Name}_hal_library
cd ..

activate.${CPURank}_cpu_stm32w1xx.sh  QUIET "\$0"
activate.500_ttc_string.sh QUIET "\$0"  # ttc_string_printf() is required

END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}
    #}_hal
    Name="${LibsRank3}_${INSTALL_PREFIX}_std_peripherals" #{ makefile + activate-script for STM Standard Peripherals Library
    createExtensionMakefileHead ${Name} #{
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File #{

# include files of STM standard peripherals library
INCLUDE_DIRS += -I additionals/${LibsRank3}_${INSTALL_PREFIX}_std_peripherals/inc/

# source files of STM standard peripherals library
vpath %.c additionals/${LibsRank3}_${INSTALL_PREFIX}_std_peripherals/src/

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{
    createActivateScriptHead $File $0 "STM32W1xx Hardware Abstraction Layer library (HAL)"
    cat <<END_OF_ACTIVATE >>$File #{

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

cd "\$Dir_Additionals"
createLink $INSTALLPATH/stm_std_peripherals/ ${Name}
cd ..

activate.${CPURank}_cpu_stm32w1xx.sh  QUIET "\$0"
activate.500_ttc_string.sh QUIET "\$0"  # ttc_string_printf() is required

END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}
    #}_std_peripherals
    
    # create permanent symbolic links to source-directories
    addLine ../scripts/createLinks.sh "rm 2>/dev/null ${CPURank}_${INSTALL_PREFIX};                      ln -sv \$HOME/Source/ToolChain_STM32/ToolChain/${CPURank}_${INSTALL_PREFIX}/SimpleMac-Firmware  ${CPURank}_${INSTALL_PREFIX}"
    addLine ../scripts/createLinks.sh "rm 2>/dev/null ${LibsRank1}_${INSTALL_PREFIX}_SimpleMAC_library;  ln -sv \$HOME/Source/ToolChain_STM32/ToolChain/${CPURank}_${INSTALL_PREFIX}/SimpleMac-Library   ${LibsRank1}_${INSTALL_PREFIX}_SimpleMAC_library"
    addLine ../scripts/createLinks.sh "rm 2>/dev/null ${LibsRank2}_${INSTALL_PREFIX}_hal_library;        ln -sv \$HOME/Source/ToolChain_STM32/ToolChain/${CPURank}_${INSTALL_PREFIX}/hal_library         ${LibsRank2}_${INSTALL_PREFIX}_hal_library"
    addLine ../scripts/createLinks.sh "rm 2>/dev/null ${LibsRank3}_${INSTALL_PREFIX}_std_peripherals;    ln -sv \$HOME/Source/ToolChain_STM32/ToolChain/${CPURank}_${INSTALL_PREFIX}/stm_std_peripherals ${LibsRank3}_${INSTALL_PREFIX}_std_peripherals"

    echo "Installed successfully: ${CPURank}_${INSTALL_PREFIX}"
  fi #}
  cd ..
  rmdir 2>/dev/null $INSTALLPATH  # removes only empty install dir
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
