#!/bin/bash
#                 ToolChain_STM32
#
#  Install script for CodeSourcery G++ compiler toolchain 
#  for CortexM3 microcontrollers from CodeSourcery.
#  Script written by Gregor Rebel 2010-2012.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="050_compiler_sourcery_gpp"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

Folder="arm-2010q1"
BinFolder="`pwd`/$Folder/bin"
ToolChainBinFolder="Source/ToolChain_STM32/ToolChain/bin"
GenericFolder="\$HOME/Source/ToolChain_STM32/ToolChain/$INSTALLPATH/$Folder/bin"

if [ ! -e OK.Install ]; then #{

echo "installing in $INSTALLPATH ..."

Archive="arm-2010q1-188-arm-none-eabi-i686-pc-linux-gnu.tar.bz2"
if [ -e $Archive ]; then
  Error=""
  bzip2 -t $Archive || Error="1"
  if [ "$Error" != "" ]; then
    echo "Archive corrupt: $Archive"
    rm -v $Archive
  fi
fi
Iterations=0
while [ "`bzip2 -t $Archive 2>&1`" != "" ]; do
  echo "trying to download $Archive ..."
  #getFile http://www.codesourcery.com/sgpp/lite/arm/portal/package6493/public/arm-none-eabi/ $Archive
  getFile http://hlb-labor.de/cs/ $Archive
  Iterations=$(( $Iterations + 1 ))
  if [ $Iterations -gt 10 ]; then
    break
  fi
done


untgj Folder $Archive
# add2CleanUp ../cleanup.sh "rm -Rf $INSTALLPATH/$Folder"

#getFile http://www.codesourcery.com/sgpp/lite/arm/portal/package6492/public/arm-none-eabi/ arm-2010q1-188-arm-none-eabi.src.tar.bz2
#untgj arm-2010q1-188-arm-none-eabi .src.tar.bz2

OldDir=`pwd`
cd ../../Documentation
dir "$INSTALLPATH"
cd "$INSTALLPATH"
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7609/ as.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7608/ binutils.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7616/ libc.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7612/ gcc.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7614/ gdb.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7615/ getting-started.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7610/ ld.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7617/ libm.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7613/ cpp.pdf
  getFile http://www.codesourcery.com/sgpp/lite/arm/portal/doc7611/ gprof.pdf
cd "$OldDir"

if [ -e "$BinFolder/arm-none-eabi-gcc" ]; then
  createExtensionMakefileHead ${INSTALLPATH} #{
  createExtensionMakefileTail ${INSTALLPATH} #}
  echo "" >OK.Install
#X  $BinFolder/arm-none-eabi-gcc --version | grep gcc
fi
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
  #X addLine ../scripts/SourceMe.sh    "export PATH=\"\$PATH:$GenericFolder/\" #$0"
  addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH;                      ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/arm-2010q1/  $INSTALLPATH"
  
  Name="${INSTALLPATH}"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

COMPILE_OPTS += -DSOURCE_COMPILER=$Name

INCLUDE_DIRS += -I additionals/$Name/arm-none-eabi/include/ \\
                -I additionals/$Name/lib/gcc/arm-none-eabi/4.4.1/include/

# binaries from CodeSourcery must be somewhere in $PATH
CC =arm-none-eabi-gcc
AR =arm-none-eabi-ar
CP =arm-none-eabi-objcopy
OD =arm-none-eabi-objdump
#LD =arm-none-eabi-ld -v
LD =arm-none-eabi-gcc                                       # workaround: floating-point bug (http://fun-tech.se/stm32/OlimexBlinky/ld_float.php)
#AS =arm-none-eabi-as
AS =\$(CC) \$(AS_OPTS) -x assembler-with-cpp -c \$(TARGET)  # workaround: .o-files compiled by as cannot be linked by ld

CFLAGS =-std=gnu99 -c -fno-common -fno-builtin -ffreestanding -funsigned-char \$(COMPILE_OPTS) \$(TARGET) 
LDFLAGS=-Wl,-Map=\$(MAIN_MAP) -nostartfiles \$(TARGET)
#X LDFLAGS += -Wl,-\\( \${filter %.a,\$^} \$(LD_LIBS) -Wl,-\\)      # workaround: make pre-compiled libraries available 
CPFLAGS=-Obinary
ODFLAGS	= -S
ASFLAGS=\$(COMPILE_OPTS)
# ASFLAGS=-gdwarf2 -mcpu=\$(ARM_CPU) -mthumb-interwork -o\$@

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" 
  createActivateScriptHead $File $0 "Compiler SourceyG++ (Closed Source, Binary only)" #{
  cat <<END_OF_ACTIVATE >>$File

rm 2>/dev/null \$Dir_ExtensionsActive/makefile.050_compiler_*

# create links into extensions.active/
createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

rm \$HOME/$ToolChainBinFolder/arm-none-eabi-*
ln -s $BinFolder/arm-none-eabi-* \$HOME/$ToolChainBinFolder/

END_OF_ACTIVATE

  createActivateScriptTail $File $Name #}
  
  #X # create symlink for compatibility to old versions of activate_project.sh <v1.0.39
  #X cd ../extensions/
  #X createLink activate.${Name}.sh activate.050_compiler_sourcery_g++.sh
  
  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
