#!/bin/bash
#
#
#  Install script for generic radio support 
#
#  Initial Script written by Gregor Rebel 2010-2012.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="500_ttc_radio"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

  Name="$INSTALLPATH"
  createExtensionMakefileHead ${Name}      #{ create makefile
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1

# append source-folder for includes + source codes
INCLUDE_DIRS += -Icommon/radio/
vpath %.c common/radio/

MAIN_OBJS += ttc_radio.o

COMPILE_OPTS += -DTTC_NETWORK_MAX_HEADER_SIZE=\$(TTC_NETWORK_MAX_HEADER_SIZE)
COMPILE_OPTS += -DTTC_NETWORK_MAX_FOOTER_SIZE=\$(TTC_NETWORK_MAX_FOOTER_SIZE)
COMPILE_OPTS += -DTTC_NETWORK_MAX_PAYLOAD_SIZE=\$(TTC_NETWORK_MAX_PAYLOAD_SIZE)

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{ create activate script
  createActivateScriptHead $File "$0" "Architecture independent support for Universal Synchronous Asynchronous Serial Receiver Transmitter (RADIO)"
  cat <<END_OF_ACTIVATE >>$File
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_Additionals/${Name}_*

Architecture=\`ls -f extensions.active/*400_radio*\`

if [ "\$Architecture" != "" ]; then

  activate.500_ttc_network.sh QUIET "\$0"
  activate.500_ttc_queue.sh   QUIET "\$0"
  activate.500_ttc_string.sh  QUIET "\$0"

  # create links into extensions.active/
  createLink "\$Dir_Extensions/makefile.$Name" "\$Dir_ExtensionsActive/makefile.$Name" '' QUIET
else
  echo "\$0 - ERROR: no supported architecture found. Activate a radio first!"
  exit 10
fi

END_OF_ACTIVATE
  createActivateScriptTail $File $Name
  #}
  echo "Installed successfully: $Name"
  
  Name="600_example_radio"
  createExtensionMakefileHead ${Name}      #{ create makefile
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1

# append some object files to be compiled
MAIN_OBJS += example_radio.o

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{ create activate script
  createActivateScriptHead $File "$0" "Example of how to use architecture independent RADIO support"
  cat <<END_OF_ACTIVATE >>$File
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_ExtensionsActive/makefile.600_example_*

activate.500_ttc_gpio.sh   QUIET \"\$0\" || exit 10
activate.500_ttc_radio.sh  QUIET \"\$0\" || exit 10
activate.500_ttc_usart.sh  QUIET \"\$0\" || exit 10
activate.500_ttc_random.sh QUIET \"\$0\" || exit 10
activate.500_ttc_queue.sh  QUIET \"\$0\" || exit 10

# create links into extensions.active/
createLink "\$Dir_Extensions/makefile.$Name" "\$Dir_ExtensionsActive/makefile.$Name" '' QUIET

END_OF_ACTIVATE
  createActivateScriptTail $File $Name
  #}
  echo "Installed successfully: $Name"
   
  Name="600_example_radio_serial"
  createExtensionMakefileHead ${Name}      #{ create makefile
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1

# append some object files to be compiled
MAIN_OBJS += example_radio_serial.o

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{ create activate script
  createActivateScriptHead $File "$0" "Example of how to use architecture independent RADIO support"
  cat <<END_OF_ACTIVATE >>$File
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_ExtensionsActive/makefile.600_example_*

activate.500_ttc_gpio.sh   QUIET \"\$0\" || exit 10
activate.500_ttc_radio.sh  QUIET \"\$0\" || exit 10
activate.500_ttc_usart.sh  QUIET \"\$0\" || exit 10
activate.500_ttc_random.sh QUIET \"\$0\" || exit 10
activate.500_ttc_queue.sh  QUIET \"\$0\" || exit 10

# create links into extensions.active/
createLink "\$Dir_Extensions/makefile.$Name" "\$Dir_ExtensionsActive/makefile.$Name" '' QUIET

END_OF_ACTIVATE
  createActivateScriptTail $File $Name
  #}
  echo "Installed successfully: $Name"

cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
