#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Initial Script written by Gregor Rebel 2010-2012.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="999_kicad"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Docs ];    then    #{ download documentation

  Get_Fails=""
  
  # download doc-file + move to Documentation/ + create symbolic link into Documentation/ right here 
  # File="STM32F10x_Standard_Peripherals_Library.html"
  # getFile http://www.mikrocontroller.net/articles/ STM32F10x_Standard_Peripherals_Library $File
  # addDocumentationFile $File STM
  
  if [ "$Get_Fails" == "" ]; then #{ check get() inside scripts/installFuncs.sh 
    touch OK.Docs
  else
    echo "$0 - ERROR: missing files: $Get_Fails"
  fi #}
fi #}
if [ ! -e OK.Install ]; then    #{ download + install software
  if [ ! -e OK.Packages ]; then #{ install packages
    # installPackage libgmp3-dev
    AptGetAvailable=`which apt-get`
    if [ "$AptGetAvailable" ]; then
      sudo add-apt-repository --yes ppa:adamwolf/kicad-testing-daily
      sudo DEBIAN_FRONTEND=noninteractive apt-get --install-suggests --force-yes --fix-missing -y update
      sudo DEBIAN_FRONTEND=noninteractive apt-get --install-suggests --force-yes --fix-missing -y install kicad kicad-doc-en
    fi
    
    touch OK.Packages
  fi #}
  if [ -e OK.Packages ];   then #{ install software
    echo "installing in $INSTALLPATH ..."
    InstallDir=`pwd`
    
    # getFile http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/ stm32_i2c_cpal.zip stm32_i2c_cpal.zip
    # unZIP "UNKNOWN" stm32_i2c_cpal.zip
    dir libs
    cd libs

    getFile http://smisioto.no-ip.org/elettronica/kicad/ kicad-en.htm
    for File in `grep -i '<img src' kicad-en.htm  | awk -F\" '{ print $2 }' | awk -F/ '{ print $5 }'`; do
      getFile http://smisioto.no-ip.org/elettronica/kicad/imgs/ $File
    done
    cp kicad-en.htm index.html
    replaceInFile index.html /elettronica/kicad/imgs/ ""
    replaceInFile index.html "bgColor=#000000" "bgColor=#e5e5e5"
    replaceInFile index.html "BACKGROUND: #000000" "BACKGROUND: #e5e5e5"
    
    for File in mod_capacitors.zip mod_conn_9159.zip mod_conn_av.zip mod_conn_d-sub.zip mod_conn_df13.zip mod_conn_jst-ph.zip mod_conn_misc.zip mod_conn_modu.zip mod_conn_panelmate.zip mod_conn_pc.zip mod_conn_screw.zip mod_conn_strip.zip mod_details.zip mod_indicators.zip mod_misc_comp.zip mod_pin_strip.zip mod_pth_circuits.zip mod_pth_diodes.zip mod_pth_plcc.zip mod_pth_resistors.zip mod_relay.zip mod_to.zip mod_smd_bga.zip mod_smd_cap.zip mod_smd_dil.zip mod_smd_diode.zip mod_smd_inductors.zip mod_smd_leds.zip mod_smd_lqfp.zip mod_smd_plcc.zip mod_smd_qfn.zip mod_smd_resistors.zip mod_smd_trans.zip; do 
      getFile http://smisioto.no-ip.org/elettronica/kicad/libs/pcbnew/ $File
    done
    for File in 3d_capacitors.zip 3d_conn_9159.zip 3d_conn_av.zip 3d_conn_d-sub 3d_conn_df13.zip 3d_conn_jst-ph 3d_conn_misc.zip 3d_conn_modu.zip 3d_conn_panelmate.zip 3d_conn_pc.zip 3d_conn_screw.zip 3d_conn_strip.zip 3d_details.zip 3d_indicators.zip 3d_misc_comp.zip 3d_pin_strip.zip 3d_pth_circuits.zip 3d_pth_diodes.zip 3d_pth_plcc.zip 3d_pth_resistors.zip 3d_relay.zip 3d_to.zip 3d_smd_bga.zip 3d_smd_cap.zip 3d_smd_dil.zip 3d_smd_diode.zip 3d_smd_inductors.zip 3d_smd_leds.zip 3d_smd_lqfp.zip 3d_smd_plcc.zip 3d_smd_qfn.zip 3d_smd_resistors.zip 3d_smd_trans.zip; do
      getFile http://smisioto.no-ip.org/elettronica/kicad/libs/3d/ $File
    done
    
    echo "" >unzip.log
    for File in *.zip; do
      echo "extracting $File..."
      unzip -o $File >>unzip.log 2>&1
    done
 
    cd ..
    
    echo "" >OK.Install
  else
    echo "$0 - ERROR: Packages were not installed correctly!"
  fi #}OK.Packages
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then    #{ create makefiles and activate scripts
  Name="${INSTALLPATH}"

  cd "$OldPWD"
  echo "Installed successfully: $INSTALLPATH"
#}
else                            #{ Install failed
  echo "failed to install $INSTALLPATH"
  exit 10
fi #}

cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
