#!/usr/bin/perl

use strict;
use File::Spec::Functions qw(rel2abs);
use File::Basename;
my $ToolChainDir = dirname(rel2abs($0));

my $ProjectFolder = shift(@ARGV);

my $ScriptName="activate_project.sh";
my $ToolChainVersion = `cat Version`;

unless ($ProjectFolder) {
  print "$0 PROJECT_FOLDER\n";
  print "Creates a new activate-script in given project folder.\n";
  print "\n";
  exit 5;
}


my $AllActivateScripts = `ls $ENV{HOME}/Source/ToolChain_STM32/ToolChain/extensions/activate.*`;
my @AllActivateScripts = split("\n", $AllActivateScripts);
@AllActivateScripts = sort { $a cmp $b; } map { my @Parts = split("/", $_); $Parts[-1]; } @AllActivateScripts;

my %Ranks;
foreach my $ActivateScript (@AllActivateScripts) {
  (my $Rank, my @Rest) = split("_", $ActivateScript);
  my $DotPos = rindex($Rank, '.');
  $Rank = substr($Rank, $DotPos+1);
  unless ($Ranks{$Rank}) {
    $Ranks{$Rank} = { 
      Scripts => [ ],        # all scripts being found in $HOME/Source/ToolChain_STM32/ToolChain/extensions/
      Comment => '',         # placed at beginning of each rank-block
      DefaultScripts => []   # list of scripts being printed as non-commented
    };
    #print "new rank: '$Rank'\n"; 
  }
  push @{ $Ranks{$Rank}->{Scripts} }, $ActivateScript;
  #X $Ranks{$Rank}->{DefaultScripts}->[0] = $ActivateScript;
}

#{ set comments + defaults
my $Rank;

$Rank = '000'; $Ranks{$Rank}->{Name}="Basic ToolChain setup"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
END_OF_COMMENT
$Ranks{$Rank}->{DefaultScripts} = ['activate.000_basic_extensions_basic_setup.sh',
                                   'activate.000_basic_extensions_optimize_1_debug.sh'
                                  ];
#}000
$Rank = '050'; $Ranks{$Rank}->{Name}="Choose C-Compiler"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT";
  #
  # Different compilers may be required for certain microcontroller architectures.

END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = ['activate.050_compiler_sourcery_gpp.sh'];

#}050
$Rank = '100'; $Ranks{$Rank}->{Name}="Board definitions (mandatory)"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #  
  # Each board 
  # - activates a CPU
  # - sets memory sizes and locations
  # - activates required standard peripheral library (STM-types only)
  #
  # Note: Exactly one board must be activated for your project!

  # activate following lines to use your own board file in extensions.local/
  # ln -s makefile.100_board_* ../extensions.local/

END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = ['activate.100_board_olimex_h107.sh'];
#}100
$Rank = '150'; $Ranks{$Rank}->{Name}="Board extensions - Addon modules (optional)"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # Any amount of board extensions may be activated at once.
  # The user is responsible to avoid double definitions and interferences.

END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}100
$Rank = '200'; $Ranks{$Rank}->{Name}="CPU"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # The CPU should already been activated by the board makefile.
  # You may want to activate a CPU in case you don't yet have a board makefile 
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}
$Rank = '250'; $Ranks{$Rank}->{Name}="Std Peripherals Library"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  # The Standard Peripherals Library has been developed by ST Microelectronics
  # as a support library for their STM32Fxxxx microcontrollers only.
  # 
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}250
$Rank = '251'; $Ranks{$Rank}->{Name}="Support Libraries for em357/stm32w108 architecture"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  # STM32Wxxx architecture (STM32W108) has been developed by Ember Corp. as EM351/EM357. 
  # These chips are not compatible with Standard Peripheral Library.
  # The HAL library can be used instead. 
  # The, binary only, SimpleMAC library uses its own HAL.
  #
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}270
$Rank = '253'; $Ranks{$Rank}->{Name}="Support Libraries for radio transceiver embedded into em357/stm32w108 architecture"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # STM32Wxxx architecture (STM32W108) has been developed by Ember Corp. as EM351/EM357. 
  # These chips are not compatible with Standard Peripheral Library.
  # The, binary only, SimpleMAC library provides an interface to make use of the radio transceiver.
  # This library is required as no register descriptions are publicly available for these microcontrollers.
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}270
$Rank = '270'; $Ranks{$Rank}->{Name}="Communication Peripheral Application Library"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # CPAL has been designed on top of std peripherals library.
  # CPAL is suitable for STM32Fxxxx microcontrollers only.
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}270
$Rank = '300'; $Ranks{$Rank}->{Name}="Multitasking scheduler"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # A multitasking scheduler provides basic multithreading features.
  # The activation of one scheduler and a heap type is mandatory for most examples. 
  #
END_OF_COMMENT
push @{ $Ranks{$Rank}->{DefaultScripts} }, 'activate.300_scheduler_free_rtos.sh',
                                           'activate.300_scheduler_free_rtos_heap1.sh';
#}
$Rank = '400'; $Ranks{$Rank}->{Name}="External Support Libraries"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = [];
#}
$Rank = '500'; $Ranks{$Rank}->{Name}="Support Libraries provided by ToolChain_STM32"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # These libraries provide example code for how to initialize
  # and use certain funtional units like watchdogs, adcs, ...
  #
END_OF_COMMENT

push @{ $Ranks{$Rank}->{DefaultScripts} }, 'activate.500_ttc_task.sh';
#}
$Rank = '600'; $Ranks{$Rank}->{Name}="Examples"; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 
  #
  # Some examples can run in parallel.
  #
  # In order to run, each example requires
  # - Board (defines TTC_LED1, TTC_LED2 other interfaces and CPU)
  # - Multitasking Scheduler
  #
  # 600_example_leds is the most simple example and should work on any board that defines TTC_LED1, TTC_LED2
  
END_OF_COMMENT

$Ranks{$Rank}->{DefaultScripts} = ['activate.600_example_leds.sh'];
#}
$Rank = ''; #{
$Ranks{$Rank}->{Comment} = <<"END_OF_COMMENT"; 

END_OF_COMMENT
#}

#}

my @Activates;
foreach my $Rank (sort { $a cmp $b} keys %Ranks) { # compile list of all available activates
  if ($Rank) {
    my $RankRef = $Ranks{$Rank};
    my $Activate = $RankRef->{Comment};
    my %DefaultScripts;
    foreach my $DefaultScript (@{ $RankRef->{DefaultScripts} }) {
      $DefaultScripts{$DefaultScript} = 1;
    }

    foreach my $ActivateScript (@{ $RankRef->{Scripts} }) {
      my $Info = `$ActivateScript INFO`;
      my @Info = split("\n", $Info);
      @Info = grep { $_ } @Info; # remove empty lines
      if (@Info) { $Activate .= "\n  # ".join("\n  # ", @Info)."\n"; }
      my $Suffix = "QUIET \"\$0\" || Error '$ActivateScript'";
      if ($DefaultScripts{$ActivateScript}) { $Activate .= "  $ActivateScript $Suffix\n"; delete $DefaultScripts{$ActivateScript}; }
      else                                  { $Activate .= "  #$ActivateScript $Suffix\n"; }
    }
    $Activate .= "\n";
    foreach my $DefaultScript (@{ $RankRef->{DefaultScripts} }) { # add remaining default scripts
      if ($DefaultScripts{$DefaultScript}) { # script not yet added: add it
        $Activate .= "  $DefaultScript QUIET \"\$0\" || Error '$DefaultScript'\n";
      }
    }

    push @Activates, 
         "if [ 1 == 1 ]; then #{ $Rank - ".$Ranks{$Rank}->{Name}."\n".
         "  Rank='$Rank'\n". # avoids syntax error in case no command is executed between if then ... fi
         $Activate.
         "\nfi #}$Rank";
  }
}
my $Activates = join("\n", @Activates);

my $Script = <<"END_OF_SCRIPT"; #{ compile content of activate-script
#!/bin/bash

#            ToolChain_STM32 v$ToolChainVersion
#
# This script configures which extensions should be activated for this project. 
#
# Each extension provides one additional makefile being linked into extensions.active/
# The extension makefile can define
# - additional constants
# - additional include directories to search for .h-files
# - additional source directories to search for .c-files
# - additional object files to create (object files get compiled from .c/.s-files)
# - other magic makefile features
#
# Minimum setup
# - 000 basic setup
# - 050 exactly one compiler
# - 100 exactly one board (be sure to set the correct!)
# - 300 exactly one scheduler (+ some scheduler settings)
# 

function age() {
   local filename=\$1
   local changed=`stat -c \%Y "\$filename"`
   local now=`date +\%s`
   local elapsed

   let elapsed=now-changed
   echo \$elapsed
}
function Error() {
  local Stage=\$1

  echo "\$0 - ERROR: project could not be fully activated at stage '\$Stage'!"
  exit 11
}

MakeFile="extensions.active/makefile"
if [ -e \$MakeFile ]; then
  AgeMakefile=\$(age \$MakeFile)
  AgeActivateProject=\$(age \$0)

  if [ \$AgeActivateProject -lt \$AgeMakefile ]; then
    echo "\$0 - project settings changed: cleaning extensions"
    ./clean.sh
  fi
fi


ExtensionsActive=`ls 2>/dev/null \${MakeFile}.*`
if [ "\$ExtensionsActive" == ""  ]; then
  echo "\$0 - no extensions active: activating configured extensions"

  # golden rule: start to activate lower ranks first:
  # activate.000_XXX.sh
  # activate.100_XXX.sh
  # ...

$Activates

  # this makefile belongs to current project and is added automatically
  cd extensions.active/
  
  # activate local makefile
  ln -s ../extensions.local/makefile.700_extra_settings .
  
  cd ..
else
  echo "\$0 - extension setup found: not changing extensions.active/"
fi
END_OF_SCRIPT
#}

if (-e $ScriptName) { system("mv $ScriptName $ScriptName.old"); }
open(OUT, ">$ScriptName") or die("$0 - ERROR: Cannot create activate-script `pwd`/$ScriptName ($!)");
print OUT $Script;
close(OUT) or die("$0 - ERROR: Cannot write to activate-script `pwd`/$ScriptName ($!)");
system("chmod +x \"$ScriptName\"");
