#!/usr/bin/perl
#
# updateProjectFiles.pl  written by Gregor Rebel 2011
#
# Automatically generates list of all source-code and header files being used 
# by current makefile-project.
#
# + How it works
#   (1) run "make all -p" to retrieve all path variables from make
#   (2) process main.c + all included files (depth-search)
#   (3) write out all processed files
#

my $ReturnCode = 0;
my $ProjectFile = shift(@ARGV);
my $FilePrefix  = shift(@ARGV);
my @Files2Process = @ARGV;

#print "$0 '$ProjectFile' '$FilePrefix'\n"; exit 0; #D
unless ($ProjectFile) {
  print "$0 PROJECT_FILE FILE_PREFIX [A.C] [B.C]..\n";
  print "  PROJECT_FILE  filename of QtCreator .files file (will be filled with all found files)\n";
  print "  FILE_PREFIX   used as prefix for each line in PROJECT_FILE\n";
  print "  A.C,B.C,..    start points of recursive search\n";
  print "\n";
  print "  $0 will run make all -p to retrieve all include- and source-paths being known to the compiler.\n";
  print "  It will then recursively scan all given .c-files for included files. All found files will be \n";
  print "  added to PROJECT_FILE.\n";
  print "\n";
  print "  Example: $0 Test123.files\n";
  print "  Example: $0 v/Test123.files ../\n";
  exit 5;
}

my $Makefile = `make all -p 2>&1`;
$Makefile =~ s/\n\r/\n/g;
$Makefile =~ s/\r\n/\n/g;
my @Makefile = split("\n", $Makefile);
$Makefile =~ s/\\\n//g;
my @ErrorMessages;

#{ extract list of c-source search paths (vpath %.c)       -> @VPathC, %VPathC
my @VPathC; my %VPathC;
my $Pos; 
my $LastPos= 0;
while ( ($Pos = index($Makefile, 'vpath %.c ', $LastPos)) > -1 ) {
  $Pos += 10;
  my $EolPos = findEndOfLine($Makefile, $Pos);
  my @NewVPathC = split(":", substr($Makefile, $Pos, $EolPos - $Pos + 1));
  @NewVPathC = map { $_ =~ s/\r//g; $_ =~ s/\n//g; $_; }  @NewVPathC;
  addEntries(\@VPathC, \%VPathC, @NewVPathC);
  $LastPos = $Pos + 1;
}
print "VPathC: ".join(',', @VPathC)."\n";
#}
#{ extract list of assembler code search paths (vpath %.s) -> @VPathS, %VPathS
my @VPathS; my %VPathS;

$LastPos = 0;
while ( ($Pos = index($Makefile, 'vpath %.s ', $LastPos)) > -1 ) {
  $Pos += 10;
  my $EolPos = findEndOfLine($Makefile, $Pos);
  my @NewVPathS = split(":", substr($Makefile, $Pos, $EolPos - $Pos + 1));
  @NewVPathS = map { $_ =~ s/\r//g; $_ =~ s/\n//g; $_; }  @NewVPathS;
  addEntries(\@VPathS, \%VPathS, @NewVPathS);
  $LastPos = $Pos + 1;
}
print "VPathS: ".join(',',@VPathS)."\n";
#}
#{ extract list of common search paths (VPATH)             -> @VPath,  %VPath
my $Pos = index($Makefile, 'Variable »VPATH«)');
my $EolPos = findEndOfLine($Makefile, $Pos);
$Pos = index($Makefile, '# ', $EolPos) + 2; # pos @ after # in next line
$EolPos = findEndOfLine($Makefile, $Pos);
my @NewVPath = split(":", substr($Makefile, $Pos, $EolPos - $Pos + 1));
@NewVPath = map { $_ =~ s/\r//g; $_ =~ s/\n//g; strip($_); }  @NewVPath;

my @VPath; my %VPath;
addEntries(\@VPath, \%VPath, '.');
print "VPath: ".join(',', addEntries(\@VPath, \%VPath, @NewVPath))."\n";
#}
#{ extract list of include paths                           -> @INCLUDE_DIRS, %INCLUDE_DIRS
my @INCLUDE_DIRS; my %INCLUDE_DIRS;
my $Pos = index($Makefile, 'INCLUDE_DIRS =');
if ($Pos > -1) {
  $Pos += 14;
  my $EolPos = findEndOfLine($Makefile, $Pos);
  my @IncludeDirs = split( " -I", substr($Makefile, $Pos, $EolPos - $Pos) );
  @IncludeDirs = map { $_ =~ s/\r//g; $_ =~ s/\n//g; $_; }  @IncludeDirs;
  unless ($IncludeDirs[0]) { shift(@IncludeDirs); }
  foreach my $IncludeDir (@IncludeDirs) {
    my %RemoveChar = (' ' => 1, '/' => 1);
    while ( $RemoveChar{substr($IncludeDir, -1)}) { substr($IncludeDir, -1) = undef; }
  }
  addEntries(\@INCLUDE_DIRS, \%INCLUDE_DIRS,  @IncludeDirs);
  print "INCLUDE_DIRS: ".join(',', @INCLUDE_DIRS)."\n";
}
#}
#{ extract list of main objects                            -> @MainObjects, %MainObjects

my @MainObjects; my %MainObjects;
my %ObjectFiles =  map { split(": ", $_); }  # "a.o: path/a.c" -> ("a.o", "path/a.c") 
                   grep { my $Pos = index($_, '.o: '); ($Pos > -1) && (substr($_, $Pos-1,1) ne '%'); } # find all entries containing '.o ' but not '%.o '
                   @Makefile;          
foreach my $Key (keys %ObjectFiles) {
  my $File = $ObjectFiles{$Key};
  my @Files = split(" ", $File), $File; # sometimes a file is printed twice in a line, serated by a space
  foreach my $SingleFile (@Files) {
    if (-e $SingleFile) {
      $ObjectFiles{$Key} = $SingleFile;
    }
    else { delete $ObjectFiles{$Key}; }
  }
} 
foreach my $Key (sort keys %ObjectFiles) {
     print "'$Key' -> '$ObjectFiles{$Key}'\n";
}

my $PATTERN='MAIN_OBJS =';
my $Pos = index($Makefile, $PATTERN);
if ($Pos > -1) {
  $Pos += length($PATTERN);
  my $EolPos = findEndOfLine($Makefile, $Pos);
  my @Objects = split( " ", substr($Makefile, $Pos, $EolPos - $Pos) );
  print "MainObjects: ".join(',', addEntries(\@MainObjects, \%MainObjects, @Objects))."\n";
}

#{ check @MainObjects for doubles

my @Doubles;
map { if ($MainObjects{$_} > 1) { push @Doubles, $_; } } @MainObjects;
if (@Doubles) {
  push @ErrorMessages, map { "Object file $_ given ".$MainObjects{$_}." times!",
                       map { "    $_"; } split("\n", `grep -Hn $_ extensions.active/makefile`);
  } @Doubles;
  push @ErrorMessages, "Check your makefiles for MAIN_OBJS+= lines!"
}
#}
unless (@Files2Process)  # no source-files given: use main-objects  
{ @Files2Process = map { if ($ObjectFiles{$_}) { $ObjectFiles{$_} }        # object-file is already known: use it  
                         else                  { substr($_, 0, -1).'c'; }  # object-file is unknown: assume it will be compiled from a .c-file
                       } @MainObjects; }

#}
#{ extract list of variables                               -> @Variables, %Variables

my @VariableLines = grep { index($_, ' = ') > -1; } @Makefile;
my @Variables;
my %Variables;

map { 
  (my $Name, my $Value) = split(" = ", $_);
  $Variables{$Name} = $Value;
} @VariableLines;
#print "Variables:\n    ".join("\n    ", map { $_.' = "'.$Variables{$_}.'"';} sort keys %Variables)."\n"; #D

# print "COMPILE_OPTS=".resolveVariable("COMPILE_OPTS")."\n";

sub resolveVariable { # recursivley determines value of given variable
  my $Name = shift;

  my $Value = $Variables{$Name};
  my $ResolvedValue;  
  my $LastPos = 0;
  my $Pos = index($Value, '$(', $LastPos);
  while ($Pos > -1) { # scan through $Value and recursively resolve all occurences of $(foo)
      $ResolvedValue .= substr($Value, $LastPos, $Pos - $LastPos);
      my $ClosePos = index($Value, ')', $Pos + 2);
      $ResolvedValue .= resolveVariable(substr($Value, $Pos + 2, $ClosePos - $Pos - 2));

      $LastPos = $ClosePos + 1;
      $Pos = index($Value, '$(', $LastPos);
  }
  $ResolvedValue .= substr($Value, $LastPos);
  #X print "resolveVariable($Name)='$ResolvedValue'\n"; #D
  
  return $ResolvedValue;
}

#}
#{ extract list of compile options                         -> @CompileOpts, %CompileOpts
my @CompileOpts; my %CompileOpts;

my $Pos = index($Makefile, 'COMPILE_OPTS =');
#X my $CompileOpts = substr($Makefile, $Pos, index($Makefile, "\n", $Pos) - $Pos);
my $CompileOpts = resolveVariable("COMPILE_OPTS");
$CompileOpts = substr($CompileOpts, index($CompileOpts, "-D"));

my @List = split(" ", $CompileOpts);
foreach my $CompileOpt (@List) {
  my ($Key,$Value) = split("=", $CompileOpt);
  if (substr($Key, 0, 2) eq '-D') { # only interested in entries starting with '-D'
    $Key = substr($Key, 2); # remove leading -D
    while (substr($Key, -1)   eq ' ') { substr($Key, -1)   = ''; }
    while (substr($Value, -1) eq ' ') { substr($Value, -1) = ''; }
    if ($Key) {
      push @CompileOpts, $Key;
      $CompileOpts{$Key} = $Value;
    }
  }
}

#}compile options

my @FilesProcessed;
my %FilesProcessed;
my @FilesFound;
my %FilesFound;
my %FilesNotFound;

my %ProjectFiles;
if (1) { # read in additional entries from exiting .files2 file
  my $ProjectFile2 = $ProjectFile.'2';
  if (! -e $ProjectFile2) { # 2-file missing: create default one
    my $Error;
    open(OUT, ">$ProjectFile2") or die("$0 ERROR: Cannot create additionals file '$ProjectFile2' ($!)\n");
    print OUT <<"END_OF_FILE";
// additional files for $ProjectFile
// All files listed here are being added to $ProjectFile on every run of $0.
//
// Lines starting with // will be ignored

makefile
extensions.active/makefile
activate_project.sh
extensions.local/makefile.700_extra_settings
extensions.local/activate.100_board_XXX.sh
extensions.local/makefile.100_board_XXX
configs/interface.cfg
compile_options.h

END_OF_FILE
    close(OUT);
  }
  if (-e $ProjectFile2) {
    open(IN, "<$ProjectFile2") or die("$0 ERROR: Cannot read from project file '$ProjectFile2' ($!)\n");
    my @Lines = <IN>;
    close(IN);
    foreach my $File (@Lines) {
      my $Entry = strip($File);
      if ( (substr($Entry,0,2) ne '//') && ($Entry ne '') ) 
        { $ProjectFiles{$Entry} = 1; }
    }
  }
}
if (@ErrorMessages) {
  print STDERR map { "$0 - ERROR: $_\n"; } @ErrorMessages;
  @ErrorMessages = ();
  $ReturnCode = 10;
}

print "\nstart processing c-files for MainObjects..\n";
processFiles(0, '', @Files2Process);

if (1) { # write out list of all active compile options
  open(OUT, ">compile.options") or die("$0 ERROR: Cannot write to project file '`pwd`/compile.options' ($!)\n");
  print OUT join("\n", map { $_.'='.$CompileOpts{$_}; } sort @CompileOpts)."\n";
  close(OUT);
}

# write out list of all active compile options as includable header file
open(OUT, ">compile_options.h") or die("$0 ERROR: Cannot write to project file '`pwd`/compile.options' ($!)\n");
print OUT <<"END_OF_HEAD"; #{
#ifndef TTC_COMPILE_OPTS_H
#define TTC_COMPILE_OPTS_H

/* This file contains all compile options gathered from makefiles during current compile run.
 * This file was created by $0.
 *
 * You may include this file from every source to provide constant definitions to your IDE (e.g. QtCreator/ Eclipse).
 *
 * DO NOT CHANGE ANYTHING HERE, IT WILL GET LOST!
 */

// undefining everything to avoid tons of redefinition warnings 
END_OF_HEAD
#}
print OUT join("\n", map { 

"  #ifdef $_\n".    
"  #undef $_\n".
"  #endif"; 

} sort @CompileOpts)."\n";
print OUT <<"END_OF_HEAD";

// definitions collected from makefiles 
END_OF_HEAD
print OUT join("\n", map { 
    if ($CompileOpts{$_} ne undef) { '  #define '.$_.' '.$CompileOpts{$_}.''; }
    else                           { '  #define '.$_.' 1'; }
} sort @CompileOpts)."\n";
print OUT <<"END_OF_HEAD";

#endif // TTC_COMPILE_OPTS_H
END_OF_HEAD
close(OUT);

# write out configured memory layout

my $OutFile="configs/memory.ld";
my $Error;
open(OUT, ">$OutFile") or $Error = "$!"; 
if ($Error) { my $PWD=`pwd`; chomp($PWD); die("$0 ERROR: Cannot write to linker script '$PWD/$OutFile' ($Error)\n"); }
my $Prefix = "TTC_MEMORY_";

my @Names = grep { $_; }
            map { my $A = substr($_, length($Prefix)); my $UnderScorePos = index($_, '_'); if ($UnderScorePos > -1) { substr($A, 0, $UnderScorePos);} else { ""; } }
            grep { (index($_, $Prefix) > -1) }
            keys %CompileOpts;
my @Regions;
my %Regions;
foreach my $Name (@Names) {
  unless ($Regions{$Name}) {
    $Regions{$Name} = {};
    push @Regions, $Name;
  }
}
print "\nMemory Regions found in Makefile: ".
      join(', ', @Regions)."\n";
    
foreach my $Region (@Regions) {
  $Regions{$Region} = {
    Size   => ($CompileOpts{$Prefix.uc($Region).'_SIZE_K'} + 0).'K',
    Start  => ($CompileOpts{$Prefix.uc($Region).'_START'}),
    Access => ($CompileOpts{$Prefix.uc($Region).'_ACCESS'})
  };
  
  unless ($Regions{$Region}->{Start})  { $Regions{$Region}->{Start} = "0x00000000"; }
  unless ($Regions{$Region}->{Access}) { $Regions{$Region}->{Start} = "rxw"; }
}

my $Memories;
foreach my $Region (@Regions) {
  $Memories .= "  ".lc($Region)." (".$Regions{$Region}->{Access}.") : ORIGIN = ".$Regions{$Region}->{Start}.",  LENGTH = ".$Regions{$Region}->{Size}."\n";  
}

print OUT <<"END_OF_MEMORY"; #{ memory layout derived from defines 
/* Define memory spaces.
 *
 * Memory layout automatically created by $0
 */
 
MEMORY
{
$Memories
}

END_OF_MEMORY
close(OUT); #}

# append some files to list of project files
my @ConfigFiles = split("\n", `ls configs/`);
foreach my $ConfigFile (@ConfigFiles) { $ProjectFiles{"configs/$ConfigFile"} = 1; }
$ProjectFiles{makefile} = 1;
if (-e "main.elf.S") { $ProjectFiles{"main.elf.S"} = 1; }

# write out list of all files belonging to this project 
open(OUT, ">$ProjectFile") or die("$0 ERROR: Cannot write to project file '$ProjectFile' ($!)\n");
print OUT join("\n", map { $FilePrefix.$_; } sort keys %ProjectFiles)."\n";
close(OUT);


exit $ReturnCode;

sub processFiles() {           # scan given files for includes
  my $RecursionDepth = shift;   # increased by every recursion
  my $IncludedBy     = shift;   # name of file which included @Files2Process
  my @Files2Process = @_;
  my $Spaces;
  for (my $I = 0; $I < $RecursionDepth; $I++) { $Spaces .= '  '; }
  
  foreach my $File2Process (@Files2Process) {        # scan for all included files
    @ErrorMessages = ();
    my $FilePath = findFile($File2Process);
    unless ($FilePath) {
      if ($RecursionDepth > 0) 
        { print $Spaces."2+ $IncludedBy #include $File2Process - ERROR: ".shift(@ErrorMessages)."!\n"; }
      else
        { print "$File2Process - ERROR: ".shift(@ErrorMessages)."!\n"; }
    }
    else {
      if ( addEntries(\@FilesProcessed, \%FilesProcessed, $FilePath) ) {
        if (open(IN, "<$FilePath")) {
          $ProjectFiles{ $FilePath } = 1;
          if ($RecursionDepth > 0)
            { print $Spaces."3+ $IncludedBy: #include $File2Process (-> $FilePath)\n"; }
          else
            { print "1+ $File2Process ($FilePath)\n"; }
          my @Lines = <IN>;
          close(IN);
          my $Lines = join("", @Lines);
          if (0) { # concatenate multi-lines (corrupts line-number calculation)
            $Lines =~ s/\n\r/\n/g;
            $Lines =~ s/\r\n/\n/g;
            $Lines =~ s/\\\n//g;
          }
          
          my $Pos = index($Lines, '#include ');
          while ($Pos > 0) {
            $Pos += 9;
            my $StartPos = findNext($Lines, $Pos, '<', '"') + 1;
            my $EndPos;
            my $StartChar = substr($Lines, $Pos, 1);
            if ($StartChar eq '<') {
              $EndPos = findNext($Lines, $Pos + 1, '>') - 1;
            }
            if ($StartChar eq '"') {
              $EndPos = findNext($Lines, $Pos + 1, '"') - 1;
            }
            if ($EndPos > $StartPos) { # found include filename
              my $IncludeName = substr($Lines, $StartPos, $EndPos - $StartPos + 1);
              my $LineNo = getLineNo($Lines, $StartPos);

              if ( addEntries(\@FilesFound, \%FilesFound, $IncludeName) ) {
                # print $Spaces."| new include '$IncludeName'\n";
                processFiles($RecursionDepth+1, $File2Process.':'.$LineNo, $IncludeName);
              }
            }
          
            $Pos = index($Lines, '#include ', $Pos);
          }
        }
        else { print "$0 - ERROR: Cannot read from file '$FilePath'"; }
      }
    }
  }
  if (0) { print "DEBUG: \n'".join("'\n'", keys %ProjectFiles)."\n"; }
}

my %fF_Folders;
my %fF_FilePaths;
sub findFile() {               # finds relative or absolute file-path for given filenames
  my @FileNames = @_;
  
  my @FilePaths;
  
  foreach my $FileName (@FileNames) {
    unless (defined $fF_FilePaths{$FileName}) { # file-name new: search in all corresponding directories
      my @CorrespondingDirectories = @VPath;
      my $Suffix = getSuffix($FileName);
      if ($Suffix eq 's') { push @CorrespondingDirectories, @VPathS; }
      if ($Suffix eq 'c') { push @CorrespondingDirectories, @VPathC; }
      if ($Suffix eq 'h') { push @CorrespondingDirectories, @INCLUDE_DIRS; }
      
      foreach $CorrespondingDirectory (@CorrespondingDirectories) {
        my $FilePath = $CorrespondingDirectory.'/'.$FileName;
        
        if ( -e  $FilePath) { $fF_FilePaths{$FileName} = $FilePath; break; }
      }
      unless ($fF_FilePaths{$FileName}) { # ERROR: filepath not found
        unless ($FilesNotFound{$FileName}) {
          push @ErrorMessages, "Cannot find $FileName (.$Suffix) in any of ".
                               join(' ', @CorrespondingDirectories)."\n";
        }
        $FilesNotFound{$FileName} = 1;
      }
      #X else { $ProjectFiles{ $fF_FilePaths{$FileName} } = 1; }
    }
    push @FilePaths, $fF_FilePaths{$FileName};
  }
  
  if (wantarray) { return @FilePaths; }
  else           { return $FilePaths[0]; }
}
sub findNext() {               # finds next occurence of one of given list of characters
  my $Text       = shift;
  my $Position   = shift;
  my @Chars2Find = @_;
  
  my $Length = length($Text);
  my %Chars2Find;
  foreach my $Char2Find (@Chars2Find)
  { $Chars2Find{$Char2Find} = 1; }
  
  while ( ( ! $Chars2Find{ substr($Text, $Position , 1) } ) &&
          ($Position < $Length)
        )
        { $Position++; }
  
  return $Position;
}
sub findStartOfLine() {        # returns position of first  non-white character in current line
  my $Text     = shift;
  my $Position = shift;
  my $C;
  
  do {
    $C = substr($Text,  $Position, 1);
    if ( ($C ne "\n") && ($C ne "\r") ) { $Position--; }
  } while ( ($Position > 0) && ($C ne "\n") && ($C ne "\r") );
  my %WhiteChar = ( " " => 1, "\t" => 1 );
  
  while ( $WhiteChar{substr($Text,  $Position, 1)} && ($Position > 1) ) 
  { $Position++; }             # remove leading spaces
  
  return $Position;
}
sub findEndOfLine() {          # returns position of last non-white character before end of line
  my $Text     = shift;
  my $Position = shift;
  my $C;
  
  my $Length = length($Text);
  do {
    $C = substr($Text,  $Position, 1);
    if ( ($C ne "\n") && ($C ne "\r") ) { $Position++; }
  } while ( ($Position < $Length) && ($C ne "\n") && ($C ne "\r") );
  my %WhiteChar = ( " " => 1, "\t" => 1 );
  
  while ( $WhiteChar{substr($Text,  $Position, 1)} && ($Position > 1) ) 
  { $Position--; }             # remove trailing spaces
  
  return $Position;
}
sub addEntries() {             # adds given strings to list-ref + hash-ref if new
  my $ListRef = shift;
  my $HashRef = shift;
  
  my @AddedEntries;
  while (@_) {
    my $Entry   = strip( shift(@_) );
    
    if ($Entry) {
      unless ($HashRef->{$Entry}) {
        $HashRef->{$Entry} = 1;
        push @$ListRef, $Entry;
        push @AddedEntries, $Entry;
      }
      else {
         # print "DOUBLE: $Entry\n"; #D
         $HashRef->{$Entry}++;
      }
    }
  }
  
  if (wantarray) { return @AddedEntries; }
  else           { return $AddedEntries[0]; }
}
sub getSuffix() {              # returns filename suffix
  my $FileName = shift();
  
  my $LastDotPos = rindex($FileName, '.');
  if ($LastDotPos > -1) { return lc( substr($FileName, $LastDotPos + 1) ); }
  return undef;
}
sub getLineNo() {              # returns line number of given byte position in given string
  my $Text     = shift;
  my $Position = shift;
  
  $Text = substr($Text, 0, $Position + 1);
  
  my $LineNo = 0;
  my $LfPos = index($Text, "\n");
  while ($LfPos > -1) {
    $LineNo++;
    $LfPos = index($Text, "\n", $LfPos + 1);
  }
  
  return $LineNo;
}
my %s_WhiteChar;
sub strip() {                  # removes trailing and leading whitechars from given string
  my $String = shift;
  
  unless (%s_WhiteChar) {
    $s_WhiteChar{"\n"} = 1;
    $s_WhiteChar{"\r"} = 1;
    $s_WhiteChar{"\t"} = 1;
    $s_WhiteChar{" "}  = 1;
  }
  while ( $s_WhiteChar{substr($String, 0, 1)} ) { substr($String, 0, 1) = undef; }
  while ( $s_WhiteChar{substr($String, -1, 1)} )   { substr($String, -1)   = undef; }
  
  return $String;
}

exit 0;
